/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _MESSAGES_H
#define _MESSAGES_H

#include <iostream>
#include <fstream>
#include <string>

#include "../spiffaslave/file_data.h" 
#include "../spiffa/frontend.h"

using namespace std;

enum { SL_ALIVE,                  // messages from slave to spiffa
       SL_POINT,
       SL_PATH,
       SL_FILE,
       SL_CONF_START,
       SL_SOURCE,
       SL_OMIT,
       SL_USER,
       SL_MOUNT_POINT,
       SL_DESTINATION,
       SL_COMPRESSION,
       SL_KEEP_NUMBER,
       SL_SAVE_EXISTING_FILES,
       SL_RESTORE_ORPHANS,
       SL_SH_PN,
       SL_CAT_PN,
       SL_GZIP_PN,
       SL_BZIP2_PN,
       SL_LN_PN,
       SL_MTAB_PN,
       SL_BROWSER_PN,
       SL_LOG_TYPE,
       SL_CONF_DONE,
       SL_SAVE_DONE,
       SL_NO_PERMISSION,
       SL_DONE_BACKUP,
       SL_DONE_INCREMENTAL_BACKUP,
       SL_DONE_RESTORE,
       SL_FATAL_ERROR,
       SL_TEXT,
       SL_QUIT,

       SP_START_SOURCES,         // messages from spiffa to slave
       SP_SOURCE,
       SP_START_OMISSIONS,
       SP_OMIT,
       SP_START_USERS,
       SP_USER,
       SP_MOUNT_POINT,
       SP_DESTINATION,
       SP_COMPRESSION,
       SP_KEEP_NUMBER,
       SP_SAVE_EXISTING_FILES,
       SP_RESTORE_ORPHANS,
       SP_SH_PN,
       SP_CAT_PN,
       SP_GZIP_PN,
       SP_BZIP2_PN,
       SP_LN_PN,
       SP_MTAB_PN,
       SP_BROWSER_PN,
       SP_LOG_TYPE,
       SP_CONF_DONE,
       SP_BACKUP,
       SP_UPDATE,
       SP_START_RESTORE_SELECTION,
       SP_RESTORE_SELECTION_NAME,
       SP_END_RESTORE_SELECTION,
       SP_RESTORE_FROM,
       SP_LOAD,
       SP_QUIT,

       MBUFSIZE=2048};

struct spiffa_msg
  {
    int mtype;          // type of received/sent message
    int dtype;                // type of data in the message
    char data[MBUFSIZE];      // message data (i.e. text or whatever)
  };

// Used by both spiffa and spiffaslave
class Messages
{
 public:
  Messages(int q, int mt);
  ~Messages(void);

  bool send_message(int dtype, void *data);


 private:   // cannot be accessed outside Messages

 protected: // Can be inherited, but can't be accessed from outside the class
  int qid;
  int mstype;

};

// The version of Messages used by spiffaslave
class SlaveMessages:public Messages
{

public:
  friend class Backup;
  friend class Restore;

  SlaveMessages(int q, int mst, int mrt, string *log_type);
  ~SlaveMessages(void);

  void get_messages(void);
  void put_slaveconf(class Slaveconf *sc);
  void put_error_fp(fstream *fp);
  void log_message(string &msg);
  void write_to_errors(string &msg);
  void process_warning(string &msg);
  void process_error(string &msg);
  void process_fatal_error(string &msg);
  int get_num_errors(void);
  int quit_flag;              

// 0=not quitting, 1=normal quit (lost spiffa), 2=force quit

private:
    bool start_action(char *action);
    void finish_action(char *action);

    int mrtype;
    class Slaveconf *sconf;
    fstream log_fp;
    bool want_log;
    fstream *error_fp;
    int num_warnings;
    int num_errors;
    int num_names;
    int num_skipped;
    time_t start_time;
  
  list<File_Data> restore_selection;
//    list<string> restore_selection;
    
protected:

};

// The version of Messages used by spiffa
class SpiffaMessages:public Messages 
{
public:
  SpiffaMessages(int q, int mst, int mrt, class Spiffa *s);
  ~SpiffaMessages(void);

  void get_messages(void);
  void set_frontend(class Frontend *frontend);
  void send_mtype(int type);

private:
  int mrtype;
  class Spiffa *spiffa;
  class Frontend *f;

protected:

};

#endif // _MESSAGES_H
