/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>

#include "complain.h"

using namespace std;



Complain::Complain(void)
{
//  cout << "Complain:\n";
}
Complain::~Complain(void)
{
//  cout << "~Complain:\n";
if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

void Complain::end_complain (GtkWidget *widget, Complain *c)
{
//  cout << "end_complain: ENTERED\n";
gtk_widget_destroy (c->dialog_window);
}

void Complain::clear_complain(GtkWidget *widget, Complain *c)
{
  //  cout << "clear_complain: ENTERED\n";
  gtk_grab_remove (widget);
  delete(c);
}

/***********************************************
 * Complain(n, title, msg)                                        *
 * Displays a requester.                                           *
 * (n = 1) gives OK or (n = 2) RETRY and ABORT.   *
 * or (n=-1) gives only CONTINUE gadget.             *
 * (n = 0) gives only ABORT gadget.                       *
 * Returns TRUE (OK gadget) or FALSE.                 *
 ***********************************************/

void Complain::complain (int n, char *title, char *msg, 
       void (*YesFunc)(GtkWidget *widget, Complain *c), 
       void (*NoFunc)(GtkWidget *widget, Complain *c),
       gpointer d )
{
GtkWidget *label;
GtkWidget *button;

char *lb, *rb;

data = d;

switch (n)
  {

  case CN_ABORT:
  lb = NULL;
  rb = "ABORT";
  break;

  case CN_CONTINUE:
  lb = "CONTINUE";
  rb = NULL;
  break;

  case CN_OK:
  lb = "OK";
  rb = NULL;
  break;
  
  case CN_CANCEL:
  lb = NULL;
  rb = "CANCEL";
  break;
  
  case CN_OK_CANCEL:
  lb = "OK";
  rb = "CANCEL";
  break;

  case CN_OK_ABORT:
  lb = "OK";
  rb = "ABORT";
  break;

  case CN_RETRY_ABORT:
  lb = "RETRY";
  rb = "ABORT";
  break;

  case CN_RETRY_CANCEL:
  lb = "RETRY";
  rb = "CANCEL";
  break;

  case CN_YES_NO:
  default:
  lb = "YES";
  rb = "NO";
  break;
  }

/* --- Create a dialog window --- */
dialog_window = gtk_dialog_new ();

gtk_signal_connect (GTK_OBJECT (dialog_window), "destroy",
              GTK_SIGNAL_FUNC (clear_complain),
              this);

/* --- Set the title and add a border --- */
gtk_window_set_title (GTK_WINDOW (dialog_window), title);
gtk_container_border_width (GTK_CONTAINER (dialog_window), 0);
gtk_window_set_modal(GTK_WINDOW(dialog_window), TRUE);

pixbuf = gtk_widget_render_icon(GTK_WIDGET(dialog_window),
            GTK_STOCK_DIALOG_QUESTION,
            GTK_ICON_SIZE_DIALOG,
            "Q" );
if (pixbuf) gtk_window_set_icon( GTK_WINDOW(dialog_window), pixbuf);

/* --- Create an LEFT button (with the focus if no right button) --- */
if (lb)
  {
  button = gtk_button_new_with_label ( lb );

  if (YesFunc)
    {
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
        GTK_SIGNAL_FUNC (YesFunc),
        this);
    }
  else
    {
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
          GTK_SIGNAL_FUNC (end_complain),
          this);
    }

  /* --- Default the "Ok" button --- */
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
          button, TRUE, TRUE, 0);
  if(!rb)
    {
      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
      gtk_widget_grab_default (button);
    }

  gtk_widget_show (button);
  }

/* --- Create a RIGHT button with the focus --- */
if (rb)
  {
  button = gtk_button_new_with_label ( rb );

  if (NoFunc)
    {
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
        GTK_SIGNAL_FUNC (NoFunc),
        this);
    }
  else
    {
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
        GTK_SIGNAL_FUNC (end_complain),
        this);
    }

  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
              button, TRUE, TRUE, 0);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);
  }


/* --- Create a descriptive label --- */
label = gtk_label_new (msg);

/* --- Put some room around the label text --- */
gtk_misc_set_padding (GTK_MISC (label), 10, 10);

/* --- Add label to designated area on dialog --- */
gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox), 
              label, TRUE, TRUE, 0);

/* --- Show the label --- */
gtk_widget_show (label);


/* --- Show the dialog --- */
gtk_widget_show (dialog_window);

// Make sure the dialog keeps the focus
gtk_grab_add(dialog_window);
gtk_widget_grab_focus(dialog_window);
}
