/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <list>
#include <string>
#include <pwd.h>
#include <dirent.h>
#include <sched.h>

#include "frontend.h"
#include "busy.h"
#include "complain.h"
#include "../common/misc.h"
#include "spiffa.h"
#include "../common/messages.h"
#include "../common/tfile.h"
#include "../common/version.h"

using namespace std;
using namespace misc;

// Only a few of these are used, but keep them for now
GdkColor Frontend::red =    {0, 0xffff, 0x0000, 0x0000};
GdkColor Frontend::blue =   {0, 0x0000, 0x0000, 0xffff};
GdkColor Frontend::green =  {0, 0x0000, 0x5555, 0x0000};  // dark
GdkColor Frontend::yellow = {0, 0xffff, 0xffff, 0x0000};
GdkColor Frontend::purple = {0, 0xffff, 0x0000, 0xffff};
GdkColor Frontend::orange = {0, 0xffff, 0x9999, 0x0000};
GdkColor Frontend::cyan =   {0, 0x0000, 0xffff, 0xffff};
GdkColor Frontend::black =  {0, 0x0000, 0x0000, 0x0000};
GdkColor Frontend::white =  {0, 0xffff, 0xffff, 0xffff};


GtkWidget *Frontend::sh_pn_entry; 
GtkWidget *Frontend::cat_pn_entry; 
GtkWidget *Frontend::gzip_pn_entry; 
GtkWidget *Frontend::bzip2_pn_entry; 
GtkWidget *Frontend::ln_pn_entry;
GtkWidget *Frontend::mtab_pn_entry;
GtkWidget *Frontend::browser_pn_entry; 

GtkWidget *Frontend::action_label;
GtkWidget *Frontend::point_label;
GtkWidget *Frontend::path_label;
GtkWidget *Frontend::file_label;
GtkWidget *Frontend::status_label;
GtkWidget *Frontend::status_extra_label;

GtkWidget *Frontend::gui_info_label;
GtkWidget *Frontend::elapsed_time_label;
GtkWidget *Frontend::time_label;


guint Frontend::timer;
int Frontend::action;

/* GdkPixbuf RGBA C-Source image dump 1-byte-run-length-encoded */

#ifdef __SUNPRO_C
#pragma align 4 (Frontend::spiffa_pixbuf)
#endif
#ifdef __GNUC__
const guint8 Frontend::spiffa_pixbuf[] __attribute__ ((__aligned__ (4))) = 
#else
const guint8 Frontend::spiffa_pixbuf[] = 
#endif
{ ""
  /* Pixbuf magic (0x47646b50) */
  "GdkP"
  /* length: header (24) + pixel_data (4998) */
  "\0\0\23\236"
  /* pixdata_type (0x2010002) */
  "\2\1\0\2"
  /* rowstride (512) */
  "\0\0\2\0"
  /* width (128) */
  "\0\0\0\200"
  /* height (128) */
  "\0\0\0\200"
  /* pixel_data: */
  "\275\377\377\377\0\207\0\0\0\0\202\377\377\377\0\272\0\0\0\0\275\377"
  "\377\377\0\207\0\0\0\0\202\377\377\377\0\272\0\0\0\0\273\377\377\377"
  "\0\206\0\0\0\0\205\377\377\377\0\272\0\0\0\0\274\377\377\377\0\202\0"
  "\0\0\0\210\377\377\377\0\272\0\0\0\0\273\377\377\377\0\202\0\0\0\0\211"
  "\377\377\377\0\272\0\0\0\0\272\377\377\377\0\203\0\0\0\0\211\377\377"
  "\377\0\272\0\0\0\0\271\377\377\377\0\203\0\0\0\0\212\377\377\377\0\272"
  "\0\0\0\0\270\377\377\377\0\203\0\0\0\0\213\377\377\377\0\272\0\0\0\0"
  "\267\377\377\377\0\203\0\0\0\0\214\377\377\377\0\272\0\0\0\0\266\377"
  "\377\377\0\203\0\0\0\0\215\377\377\377\0\272\0\0\0\0\265\377\377\377"
  "\0\203\0\0\0\0\216\377\377\377\0\272\0\0\0\0\264\377\377\377\0\203\0"
  "\0\0\0\217\377\377\377\0\272\0\0\0\0\263\377\377\377\0\203\0\0\0\0\220"
  "\377\377\377\0\272\0\0\0\0\262\377\377\377\0\1""000\377\202\0\0\0\0\221"
  "\377\377\377\0\207\0\0\0\0\1""000\377\262\0\0\0\0\261\377\377\377\0\202"
  "000\377\223\377\377\377\0\207\0\0\0\0\202000\377\261\0\0\0\0\260\377"
  "\377\377\0\203000\377\223\377\377\377\0\207\0\0\0\0\203000\377\260\0"
  "\0\0\0\257\377\377\377\0\204000\377\223\377\377\377\0\207\0\0\0\0\204"
  "000\377\257\0\0\0\0\256\377\377\377\0\205000\377\223\377\377\377\0\207"
  "\0\0\0\0\205000\377\256\0\0\0\0\255\377\377\377\0\206000\377\223\377"
  "\377\377\0\207\0\0\0\0\206000\377\255\0\0\0\0\254\377\377\377\0\2070"
  "00\377\223\377\377\377\0\207\0\0\0\0\207000\377\254\0\0\0\0\253\377\377"
  "\377\0\204000\377\1\0\377\0\377\203000\377\223\377\377\377\0\207\0\0"
  "\0\0\203000\377\1\377\0\0\377\204000\377\253\0\0\0\0\252\377\377\377"
  "\0\204000\377\202\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\202\377\0\0\377\204000\377\252\0\0\0\0\251\377\377\377"
  "\0\204000\377\203\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\203\377\0\0\377\204000\377\251\0\0\0\0\250\377\377\377"
  "\0\204000\377\204\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\204\377\0\0\377\204000\377\250\0\0\0\0\247\377\377\377"
  "\0\204000\377\205\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\205\377\0\0\377\204000\377\247\0\0\0\0\246\377\377\377"
  "\0\204000\377\206\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\206\377\0\0\377\204000\377\246\0\0\0\0\245\377\377\377"
  "\0\204000\377\207\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\207\377\0\0\377\204000\377\245\0\0\0\0\244\377\377\377"
  "\0\204000\377\210\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\210\377\0\0\377\204000\377\244\0\0\0\0\243\377\377\377"
  "\0\204000\377\211\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\211\377\0\0\377\204000\377\243\0\0\0\0\242\377\377\377"
  "\0\204000\377\212\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\212\377\0\0\377\204000\377\242\0\0\0\0\241\377\377\377"
  "\0\204000\377\213\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\213\377\0\0\377\204000\377\241\0\0\0\0\240\377\377\377"
  "\0\204000\377\214\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\214\377\0\0\377\204000\377\240\0\0\0\0\237\377\377\377"
  "\0\204000\377\215\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\215\377\0\0\377\204000\377\237\0\0\0\0\236\377\377\377"
  "\0\204000\377\216\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\216\377\0\0\377\204000\377\236\0\0\0\0\235\377\377\377"
  "\0\204000\377\217\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\217\377\0\0\377\204000\377\235\0\0\0\0\234\377\377\377"
  "\0\204000\377\220\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\220\377\0\0\377\204000\377\234\0\0\0\0\233\377\377\377"
  "\0\204000\377\221\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\221\377\0\0\377\204000\377\233\0\0\0\0\232\377\377\377"
  "\0\204000\377\222\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\222\377\0\0\377\204000\377\232\0\0\0\0\231\377\377\377"
  "\0\204000\377\223\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\223\377\0\0\377\204000\377\231\0\0\0\0\230\377\377\377"
  "\0\204000\377\224\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\224\377\0\0\377\204000\377\230\0\0\0\0\227\377\377\377"
  "\0\204000\377\225\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\225\377\0\0\377\204000\377\227\0\0\0\0\226\377\377\377"
  "\0\204000\377\226\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\226\377\0\0\377\204000\377\226\0\0\0\0\225\377\377\377"
  "\0\204000\377\227\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\227\377\0\0\377\204000\377\225\0\0\0\0\224\377\377\377"
  "\0\204000\377\230\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\230\377\0\0\377\204000\377\224\0\0\0\0\223\377\377\377"
  "\0\204000\377\231\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\231\377\0\0\377\204000\377\223\0\0\0\0\222\377\377\377"
  "\0\204000\377\232\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\232\377\0\0\377\204000\377\222\0\0\0\0\221\377\377\377"
  "\0\204000\377\233\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\233\377\0\0\377\204000\377\221\0\0\0\0\220\377\377\377"
  "\0\204000\377\234\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\234\377\0\0\377\204000\377\220\0\0\0\0\217\377\377\377"
  "\0\204000\377\235\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\235\377\0\0\377\204000\377\217\0\0\0\0\216\377\377\377"
  "\0\204000\377\236\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\236\377\0\0\377\204000\377\216\0\0\0\0\215\377\377\377"
  "\0\204000\377\237\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\237\377\0\0\377\204000\377\215\0\0\0\0\214\377\377\377"
  "\0\204000\377\240\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0"
  "\0\203000\377\240\377\0\0\377\204000\377\214\0\0\0\0\213\377\377\377"
  "\0\204000\377\241\0\377\0\377\240000\377\241\377\0\0\377\204000\377\213"
  "\0\0\0\0\212\377\377\377\0\204000\377\242\0\377\0\377\240000\377\242"
  "\377\0\0\377\204000\377\212\0\0\0\0\211\377\377\377\0\204000\377\243"
  "\0\377\0\377\240000\377\243\377\0\0\377\204000\377\211\0\0\0\0\210\377"
  "\377\377\0\204000\377\263\0\377\0\377\202000\377\263\377\0\0\377\204"
  "000\377\210\0\0\0\0\207\377\377\377\0\204000\377\264\0\377\0\377\202"
  "000\377\264\377\0\0\377\204000\377\207\0\0\0\0\206\377\377\377\0\204"
  "000\377\265\0\377\0\377\202000\377\265\377\0\0\377\204000\377\206\0\0"
  "\0\0\205\377\377\377\0\204000\377\266\0\377\0\377\202000\377\266\377"
  "\0\0\377\204000\377\205\0\0\0\0\204\377\377\377\0\204000\377\267\0\377"
  "\0\377\202000\377\267\377\0\0\377\204000\377\204\0\0\0\0\203\377\377"
  "\377\0\204000\377\270\0\377\0\377\202000\377\270\377\0\0\377\204000\377"
  "\203\0\0\0\0\202\377\377\377\0\204000\377\271\0\377\0\377\202000\377"
  "\271\377\0\0\377\204000\377\202\0\0\0\0\1\377\377\377\0\204000\377\272"
  "\0\377\0\377\202000\377\272\377\0\0\377\204000\377\1\0\0\0\0\204000\377"
  "\273\0\377\0\377\202000\377\273\377\0\0\377\210000\377\273\0\377\0\377"
  "\202000\377\273\377\0\0\377\204000\377\1\377\377\377\0\204000\377\272"
  "\0\377\0\377\202000\377\272\377\0\0\377\204000\377\1\0\0\0\0\202\377"
  "\377\377\0\204000\377\271\0\377\0\377\202000\377\271\377\0\0\377\204"
  "000\377\202\0\0\0\0\203\377\377\377\0\204000\377\270\0\377\0\377\202"
  "000\377\270\377\0\0\377\204000\377\203\0\0\0\0\204\377\377\377\0\204"
  "000\377\267\0\377\0\377\202000\377\267\377\0\0\377\204000\377\204\0\0"
  "\0\0\205\377\377\377\0\204000\377\266\0\377\0\377\202000\377\266\377"
  "\0\0\377\204000\377\205\0\0\0\0\206\377\377\377\0\204000\377\265\0\377"
  "\0\377\202000\377\265\377\0\0\377\204000\377\206\0\0\0\0\207\377\377"
  "\377\0\204000\377\264\0\377\0\377\202000\377\264\377\0\0\377\204000\377"
  "\207\0\0\0\0\210\377\377\377\0\204000\377\263\0\377\0\377\202000\377"
  "\263\377\0\0\377\204000\377\210\0\0\0\0\211\377\377\377\0\204000\377"
  "\243\0\377\0\377\240000\377\243\377\0\0\377\204000\377\211\0\0\0\0\212"
  "\377\377\377\0\204000\377\242\0\377\0\377\240000\377\242\377\0\0\377"
  "\204000\377\212\0\0\0\0\213\377\377\377\0\204000\377\241\0\377\0\377"
  "\240000\377\241\377\0\0\377\204000\377\213\0\0\0\0\214\377\377\377\0"
  "\204000\377\240\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\240\377\0\0\377\204000\377\214\0\0\0\0\215\377\377\377\0"
  "\204000\377\237\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\237\377\0\0\377\204000\377\215\0\0\0\0\216\377\377\377\0"
  "\204000\377\236\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\236\377\0\0\377\204000\377\216\0\0\0\0\217\377\377\377\0"
  "\204000\377\235\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\235\377\0\0\377\204000\377\217\0\0\0\0\220\377\377\377\0"
  "\204000\377\234\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\234\377\0\0\377\204000\377\220\0\0\0\0\221\377\377\377\0"
  "\204000\377\233\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\233\377\0\0\377\204000\377\221\0\0\0\0\222\377\377\377\0"
  "\204000\377\232\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\232\377\0\0\377\204000\377\222\0\0\0\0\223\377\377\377\0"
  "\204000\377\231\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\231\377\0\0\377\204000\377\223\0\0\0\0\224\377\377\377\0"
  "\204000\377\230\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\230\377\0\0\377\204000\377\224\0\0\0\0\225\377\377\377\0"
  "\204000\377\227\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\227\377\0\0\377\204000\377\225\0\0\0\0\226\377\377\377\0"
  "\204000\377\226\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\226\377\0\0\377\204000\377\226\0\0\0\0\227\377\377\377\0"
  "\204000\377\225\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\225\377\0\0\377\204000\377\227\0\0\0\0\230\377\377\377\0"
  "\204000\377\224\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\224\377\0\0\377\204000\377\230\0\0\0\0\231\377\377\377\0"
  "\204000\377\223\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\223\377\0\0\377\204000\377\231\0\0\0\0\232\377\377\377\0"
  "\204000\377\222\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\222\377\0\0\377\204000\377\232\0\0\0\0\233\377\377\377\0"
  "\204000\377\221\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\221\377\0\0\377\204000\377\233\0\0\0\0\234\377\377\377\0"
  "\204000\377\220\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\220\377\0\0\377\204000\377\234\0\0\0\0\235\377\377\377\0"
  "\204000\377\217\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\217\377\0\0\377\204000\377\235\0\0\0\0\236\377\377\377\0"
  "\204000\377\216\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\216\377\0\0\377\204000\377\236\0\0\0\0\237\377\377\377\0"
  "\204000\377\215\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\215\377\0\0\377\204000\377\237\0\0\0\0\240\377\377\377\0"
  "\204000\377\214\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\214\377\0\0\377\204000\377\240\0\0\0\0\241\377\377\377\0"
  "\204000\377\213\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\213\377\0\0\377\204000\377\241\0\0\0\0\242\377\377\377\0"
  "\204000\377\212\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\212\377\0\0\377\204000\377\242\0\0\0\0\243\377\377\377\0"
  "\204000\377\211\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\211\377\0\0\377\204000\377\243\0\0\0\0\244\377\377\377\0"
  "\204000\377\210\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\210\377\0\0\377\204000\377\244\0\0\0\0\245\377\377\377\0"
  "\204000\377\207\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\207\377\0\0\377\204000\377\245\0\0\0\0\246\377\377\377\0"
  "\204000\377\206\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\206\377\0\0\377\204000\377\246\0\0\0\0\247\377\377\377\0"
  "\204000\377\205\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\205\377\0\0\377\204000\377\247\0\0\0\0\250\377\377\377\0"
  "\204000\377\204\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\204\377\0\0\377\204000\377\250\0\0\0\0\251\377\377\377\0"
  "\204000\377\203\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\203\377\0\0\377\204000\377\251\0\0\0\0\252\377\377\377\0"
  "\204000\377\202\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0"
  "\203000\377\202\377\0\0\377\204000\377\252\0\0\0\0\253\377\377\377\0"
  "\204000\377\1\0\377\0\377\203000\377\223\377\377\377\0\207\0\0\0\0\203"
  "000\377\1\377\0\0\377\204000\377\253\0\0\0\0\254\377\377\377\0\20700"
  "0\377\223\377\377\377\0\207\0\0\0\0\207000\377\254\0\0\0\0\255\377\377"
  "\377\0\206000\377\223\377\377\377\0\207\0\0\0\0\206000\377\255\0\0\0"
  "\0\256\377\377\377\0\205000\377\223\377\377\377\0\207\0\0\0\0\205000"
  "\377\256\0\0\0\0\257\377\377\377\0\204000\377\223\377\377\377\0\207\0"
  "\0\0\0\204000\377\257\0\0\0\0\260\377\377\377\0\203000\377\223\377\377"
  "\377\0\207\0\0\0\0\203000\377\260\0\0\0\0\261\377\377\377\0\202000\377"
  "\1\0\0\0\0\222\377\377\377\0\207\0\0\0\0\202000\377\261\0\0\0\0\262\377"
  "\377\377\0\2""000\377\0\0\0\0\222\377\377\377\0\207\0\0\0\0\1""000\377"
  "\262\0\0\0\0\263\377\377\377\0\202\0\0\0\0\221\377\377\377\0\272\0\0"
  "\0\0\264\377\377\377\0\203\0\0\0\0\217\377\377\377\0\272\0\0\0\0\265"
  "\377\377\377\0\202\0\0\0\0\217\377\377\377\0\272\0\0\0\0\265\377\377"
  "\377\0\203\0\0\0\0\216\377\377\377\0\272\0\0\0\0\261\377\377\377\0\203"
  "\0\0\0\0\202\377\377\377\0\205\0\0\0\0\213\377\377\377\0\272\0\0\0\0"
  "\263\377\377\377\0\2\0\0\0\0\377\377\377\0\210\0\0\0\0\211\377\377\377"
  "\0\272\0\0\0\0\265\377\377\377\0\203\0\0\0\0\1\377\377\377\0\205\0\0"
  "\0\0\210\377\377\377\0\272\0\0\0\0\266\377\377\377\0\211\0\0\0\0\207"
  "\377\377\377\0\272\0\0\0\0\270\377\377\377\0\205\0\0\0\0\211\377\377"
  "\377\0\272\0\0\0\0\273\377\377\377\0\204\0\0\0\0\207\377\377\377\0\272"
  "\0\0\0\0\275\377\377\377\0\203\0\0\0\0\206\377\377\377\0\272\0\0\0\0"
  "\276\377\377\377\0\204\0\0\0\0\204\377\377\377\0\272\0\0\0\0\276\377"
  "\377\377\0\205\0\0\0\0\203\377\377\377\0\272\0\0\0\0"};

Frontend::Frontend(int *argc, char ***argv, SpiffaMessages *msg, Spiffa *s)
{
GtkWidget *wid, *table;
GtkStyle *whitestyle;

m = msg;
m->set_frontend(this);

spiffa = s;

compression_group = NULL;
log_group = NULL;
user_treeview = NULL;   // use this to check if root...
filew = NULL;
select_dialog = NULL;
bw = NULL;
timer = 0;

/********************************

for (int n=0; n<*argc; ++n)
  {
  cout << *argv[n] << endl;
  }

********************************/


gtk_init (argc, argv);

black_text_style = create_style(black, black, white);
red_text_style = create_style(red, red, white);
blue_text_style = create_style(blue, blue, white);
green_text_style = create_style(green, purple, white);
purple_text_style = create_style(purple, purple, white);
orange_text_style = create_style(orange, orange, white);

window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_window_set_title(GTK_WINDOW(window), "spiffa");
gtk_widget_set_size_request (window, 600, 500);  // w, h
gtk_widget_show (window);
g_signal_connect(G_OBJECT(window), "delete_event",
      G_CALLBACK (callback_quit),
      NULL);
      
whitestyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(window)));
for (int i=0; i<5; i++)
    {  
    whitestyle->bg[i] = white;
    }
gtk_widget_set_style(GTK_WIDGET(window), whitestyle);

  spiffa_icon_pixbuf = gdk_pixbuf_new_from_inline(-1,
             spiffa_pixbuf,
             FALSE,
             NULL);
gtk_window_set_icon( GTK_WINDOW(window), spiffa_icon_pixbuf);

  tooltips = gtk_tooltips_new();

  wid = gtk_vbox_new(FALSE, 4);
  gtk_widget_show (wid);
  gtk_container_add (GTK_CONTAINER (window), wid);

  table = gtk_table_new(500, 600, TRUE); // 500(h) rows, 600 columns(w), homogeneous
  gtk_container_add(GTK_CONTAINER(wid), table);
  gtk_widget_show (table);

  notebook = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK(notebook), GTK_POS_TOP);
  gtk_widget_show(notebook);
  gtk_widget_set_style(GTK_WIDGET(notebook), whitestyle);


  gtk_table_attach(GTK_TABLE(table), notebook,
       10, 590, 10, 420,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding


  wid = add_page(notebook, "Actions");
  fill_backup_actions_page(wid);

  wid = add_page(notebook, "Backup Options");
  fill_backup_options_page(wid);

  wid = add_page(notebook, "Other Options");
  fill_other_options_page(wid);

  if ( STREQ( (getpwuid( getuid() ))->pw_name , "root" ) )
    {
      wid = add_page(notebook, "Users");
      fill_users_page(wid);

      wid = add_page(notebook, "Pathnames");
      fill_pathnames_page(wid);
    }

  wid = add_page(notebook, "About");
  fill_about_page(wid);

  gui_info_label = gtk_label_new("");
  gtk_widget_show(gui_info_label);
  gtk_table_attach(GTK_TABLE(table), gui_info_label,
       15, 390, 420, 440,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(gui_info_label), green_text_style);
  gtk_misc_set_alignment(GTK_MISC(gui_info_label), 0.0, 0.5);

  elapsed_time_label = gtk_label_new("");
  gtk_widget_show(elapsed_time_label);
  gtk_table_attach(GTK_TABLE(table), elapsed_time_label,
       400, 500, 420, 440,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_misc_set_alignment(GTK_MISC(elapsed_time_label), 0.0, 0.5);

  time_label = gtk_label_new("");
  gtk_widget_show(time_label);
  gtk_table_attach(GTK_TABLE(table), time_label,
       510, 585, 420, 440,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_misc_set_alignment(GTK_MISC(time_label), 1.0, 0.5);
  gtk_widget_set_style(GTK_WIDGET(time_label), blue_text_style);

  wid = gtk_button_new_with_label("SAVE");
  g_signal_connect (G_OBJECT (wid), "clicked",
          G_CALLBACK (callback_save), this);
  gtk_widget_show (wid);
  GTK_WIDGET_SET_FLAGS(wid, GTK_CAN_DEFAULT);

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "SAVE\nSave all the data to \""
      SYSCONFDIR
      "/spiffa.conf\"",
      NULL);

  gtk_table_attach(GTK_TABLE(table), wid,
       10, 110, 450, 490, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("SAVE AS");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_save_as), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "SAVE AS\n"
      "Saves the all data to a selected file",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       170, 270, 450, 490, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("LOAD");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_load), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "LOAD\n"
      "Load data to edit from selected configuration file",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       330, 430, 450, 490, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("QUIT");
  g_signal_connect (G_OBJECT (wid), "button_press_event",
          G_CALLBACK (callback_quit), this);
  gtk_widget_show (wid);

  GTK_WIDGET_SET_FLAGS(wid, GTK_CAN_DEFAULT);

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
        "QUIT\nTerminates the program.",
        NULL);

  gtk_table_attach(GTK_TABLE(table), wid,
       490, 590, 450, 490, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  gtk_main();
}

Frontend::~Frontend(void)
{
//cout << "~Frontend\n";
g_object_unref(spiffa_icon_pixbuf);
}


void Frontend::report(char *desired_action, Frontend *f)
{
//  cout << "Frontend::report\n";

if (f->timer)
  {
  string msg = "Sorry! Can't ";
  msg += desired_action;
  msg += " because spiffaslave is busy with ";

  switch (f->action)
    {
    case F_BACKUP:
    msg += "NEW BACKUP";
    break;

    case F_UPDATE:
    msg += "INCREMENTAL BACKUP";
    break;

    case F_RESTORE:
    msg += "RESTORE";
    break;

    case F_LOAD:
    msg += "LOAD CONFIGURATION";
    break;

    case F_SAVE:
    msg += "SAVE CONFIGURATION";
    break;

    case F_SAVE_AS:
    msg += "SAVE CONFIGURATION AS";
    break;
    
    case F_RESTORE_POINT:
    msg += "PATHNAME OF RESTORE POINT";
    break;
    
    case F_FILE_PATHNAME:
    msg += "PATHNAME OF FILE";
    break;

    case F_DIRECTORY_PATHNAME:
    msg += "PATHNAME OF DIRECTORY";
    break;

    case F_FILE_OR_DIRECTORY_PATHNAME:
    msg += "PATHNAME OF DIRECTORY OR FILE";
    break;

    default:
    msg += "UNKNOWN!";
    break;
    }

  gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
  }
}


void Frontend::no_quit(GtkWidget *widget, Complain *c)
{
Frontend *f = (Frontend *)c->data;
f->report("quit", f);
c->end_complain(widget, c);
return; // TRUE;
}

void Frontend::do_quit(GtkWidget *widget, Complain *c)
{
Frontend *f = (Frontend *)c->data;

c->end_complain(widget, c);

// Send quit message to spiffslave
f->m->send_message(SP_QUIT, NULL);

// Wait for message from slave that it is quitting
while(f->spiffa->quit_flag == FALSE)
  {
  f->m->get_messages();
  usleep(200);
  }

// Then quit
gtk_main_quit();      // Quits GTK
return;               // Continue destroying window
}


gint Frontend::callback_quit(GtkWidget *widget, GdkEventButton *event, Frontend *f)
{
  //  cout << "Frontend::callback_quit\n";

  if (f->timer)
    {

      string msg = "Spiffaslave is busy with:-\n";

      switch (f->action)
        {
        case F_BACKUP:
         msg += "NEW BACKUP";
        break;

        case F_UPDATE:
        msg += "INCREMENTAL BACKUP";
        break;

        case F_RESTORE:
        msg += "RESTORE";
        break;

        case F_LOAD:
        msg += "LOAD CONFIGURATION";
        break;

        case F_SAVE:
        msg += "SAVE CONFIGURATION";
        break;

        case F_SAVE_AS:
        msg += "SAVE CONFIGURATION AS";
        break;
    
        case F_RESTORE_POINT:
        msg += "PATHNAME OF RESTORE POINT";
        break;
    
        case F_FILE_PATHNAME:
        msg += "PATHNAME OF FILE";
        break;

        case F_DIRECTORY_PATHNAME:
        msg += "PATHNAME OF DIRECTORY";
        break;

        case F_FILE_OR_DIRECTORY_PATHNAME:
        msg += "PATHNAME OF DIRECTORY OR FILE";
        break;
    
        default:
        msg += "UNKNOWN!";
        break;
        }

      msg += "\nAre you sure you want to quit?";

      Complain *c = new Complain;

      c->complain(c->CN_YES_NO,
                  "Spiffa",
                  (char *)msg.c_str(),
                  do_quit,
                  no_quit,
                  (void *)f);
    }
  else
    {
      gtk_main_quit();      // Quits GTK
    }
  return TRUE;
}


GtkWidget *Frontend::add_page(GtkWidget *notebook, char *title)
{
GtkWidget *label;
GtkWidget *vbox;

vbox = gtk_vbox_new(FALSE, 0);
gtk_widget_show (vbox);

label = gtk_label_new(title);
gtk_notebook_append_page(GTK_NOTEBOOK (notebook), vbox, label);

return(vbox);
}



GtkTreeModel *Frontend::create_model(list<string> mlist, gchar *title)
{
GtkListStore *store;
GtkTreeIter iter;
list<string>::iterator i;

store = gtk_list_store_new (1, G_TYPE_STRING);

for (i=mlist.begin(); i!=mlist.end(); ++i)
  {
  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, (gchar *)i->c_str(), -1);
  }

return GTK_TREE_MODEL(store);
}

void Frontend::add_columns (GtkTreeView *treeview, gchar *title)
{
GtkCellRenderer *renderer;
GtkTreeViewColumn *column;

renderer = gtk_cell_renderer_text_new ();
column = gtk_tree_view_column_new_with_attributes(title,
                 renderer,
                 "text",
                 0,
                 NULL);
gtk_tree_view_column_set_sort_column_id (column, 0);
gtk_tree_view_append_column(treeview, column);

}

GtkWidget * Frontend::get_list_store(list<string> mlist, gchar *title)
{
GtkWidget *sw;
GtkTreeModel *model;
GtkWidget *treeview;

sw = gtk_scrolled_window_new (NULL, NULL);
gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
             GTK_SHADOW_ETCHED_IN);
gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
              GTK_POLICY_AUTOMATIC,
              GTK_POLICY_AUTOMATIC);

model = create_model(mlist, title);

treeview = gtk_tree_view_new_with_model(model);
gtk_tree_view_set_rules_hint(GTK_TREE_VIEW (treeview), TRUE);
gtk_tree_view_set_search_column(GTK_TREE_VIEW (treeview),
               0);

g_object_unref (model);

gtk_container_add (GTK_CONTAINER (sw), treeview);

/* add columns to the tree view */
add_columns(GTK_TREE_VIEW (treeview), title);

if (STREQ(title, "Sources to backup"))
  source_treeview = treeview;
else if (STREQ(title, "Sources to omit"))
  omit_treeview = treeview;
else
  user_treeview = treeview;

gtk_widget_show_all(sw);
return sw;
}


void Frontend::fill_backup_actions_page(GtkWidget *vbox)
{
  GtkWidget *wid, *table;

  table = gtk_table_new(380, 580, TRUE); // 380 rows, 580 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(vbox), table);
  gtk_widget_show (table);



  wid = gtk_label_new("S");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
         200, 210, 10, 25,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new(" ave");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
         210, 380, 10, 25,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), blue_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new("P");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
         226, 236, 25, 40,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new(" recious");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         236, 380, 25, 40,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), blue_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new("I");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         252, 262, 40, 55,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new(" nformation");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         262, 380, 40, 55,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), blue_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new("F");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         278, 288, 55, 70,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new(" or");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         288, 380, 55, 70,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), blue_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new("F");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         304, 314, 70, 85,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new(" uture");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         314, 380, 70, 85,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), blue_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new("A");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         330, 340, 85, 100,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  wid = gtk_label_new(" ccess");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         340, 390, 85, 100,
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
         0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(wid), blue_text_style);
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);

  // Busy
  rainbow_world_image_bu = gtk_image_new();
  gtk_widget_hide(rainbow_world_image_bu);
  wid = gtk_event_box_new();
  gtk_widget_show (wid);
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_event_box_set_above_child(GTK_EVENT_BOX(wid), FALSE);
  gtk_container_add(GTK_CONTAINER(wid), rainbow_world_image_bu);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         50, 100, 20, 70, 
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
         0, 0);  // xpadding, ypadding

  rainbow_world_image_rs = gtk_image_new();
  gtk_widget_hide(rainbow_world_image_rs);
  wid = gtk_event_box_new();
  gtk_widget_show (wid);
  gtk_widget_set_style(GTK_WIDGET(wid), red_text_style);
  gtk_event_box_set_above_child(GTK_EVENT_BOX(wid), FALSE);
  gtk_container_add(GTK_CONTAINER(wid), rainbow_world_image_rs);
  gtk_table_attach(GTK_TABLE(table), wid,
       // left attach, right attach, top attach, bottom attach
         480, 530, 20, 70, 
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
         0, 0);  // xpadding, ypadding

  wid = gtk_button_new_with_label("BACKUP");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT(wid), "clicked",
          G_CALLBACK (callback_backup), this);

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BACKUP\nNormally backup uses the default configuration\n\""
      SYSCONFDIR
      "/spiffa.conf\" to obtain the required\n"
      "sources, omissions, destination and compression.  If\n"
      "a different configuration file is specified then that file\n"
      "is used instead.",
      NULL);

  gtk_table_attach(GTK_TABLE(table), wid,
       10, 140, 105, 150, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("INCREMENTAL\n    BACKUP");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT(wid), "clicked",
          G_CALLBACK (callback_update), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "INCREMENTAL BACKUP\n"
      "Update the latest previous backup using the\n"
      "either the default configuration or the chosen\n"
      "configuration file if it is different.\n"
      "Changed files are updated in the backup directory\n"
      "and new files are added. Deleted files are marked\n"
      "as orphans in the backup data directory.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       200, 380, 105, 150, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("RESTORE");
  gtk_widget_show (wid);
  g_signal_connect (G_OBJECT (wid), "clicked",
          G_CALLBACK (callback_restore_from), this);

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "RESTORE\nRestore normally restores the latest backup using\n"
      "the default configuration \""
      SYSCONFDIR
      "/spiffa.conf\".\n"
      "If another configuration file is selected then that configuration\n"
      "is used instead.  If a restore point other than the latest\n"
      "backup is selected then that restore point is used instead.\n"
      "NOTE:  If selected, the restore point takes precedence over\n"
      "the backup destination in the configuration file.",
      NULL);

  gtk_table_attach(GTK_TABLE(table), wid,
       440, 570, 105, 150, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_label_new("Configuration File");
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       10, 130, 155, 195, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  config_entry = gtk_entry_new();
  gtk_widget_show(config_entry);

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), config_entry,
      "CONFIGURATION FILE\nNormally backup, incremental backup and restore use\n"
      "the default configuration \""
      SYSCONFDIR
      "/spiffa.conf\"\n"
      "to obtain the required data for the operation. If\n"
      "a different configuration file is required then enter the full\n"
      "pathname of the file to be used instead.",
      NULL);

  gtk_table_attach(GTK_TABLE(table), config_entry,
         140, 480, 155, 195, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_config), this);

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for Configuration File\n"
      "Normally backup, incremental backup and restore use\n"
      "the default configuration \""
      SYSCONFDIR
      "/spiffa.conf\"\n"
      "to obtain the required data for the operation. If\n"
      "a different configuration file is specified then that file\n"
      "is used instead.",
      NULL);

  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 155, 195, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
 
  restore_point_label = gtk_label_new("Restore Point");
  gtk_misc_set_alignment(GTK_MISC(restore_point_label), 0.0, 0.5);
  gtk_widget_show (restore_point_label);
  gtk_table_attach(GTK_TABLE(table), restore_point_label,
       10, 130, 200, 240, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  from_entry = gtk_entry_new();
  gtk_widget_show (from_entry);

  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), from_entry,
      "RESTORE POINT\n"
      "Normally restore uses the latest backup at the destination\n"
      "specified in either the default configuration file\n\""
      SYSCONFDIR
      "/spiffa.conf\" or the the selected configuration\n"
      "file.  If a different restore point is required then enter its full\n"
      "pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), from_entry,
         140, 480, 200, 240, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "button_press_event",
          G_CALLBACK(callback_browse_from), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for Restore Point\n"
      "Normally restore uses the latest backup at the destination\n"
      "specified in either the default configuration file\n\""
      SYSCONFDIR
      "/spiffa.conf\" or the the selected configuration file.\n"
      "LEFT CLICK to choose a different restore point.\n"
      "RIGHT CLICK to make a selection from the chosen or default restore point",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 200, 240, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
 
  // Report area
  wid = gtk_frame_new("");
  gtk_widget_show(wid);
  gtk_table_attach(GTK_TABLE(table), wid,
         10, 570, 240, 380,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding

  action_label = gtk_label_new("");
  gtk_widget_show(action_label);
  gtk_table_attach(GTK_TABLE(table), action_label,
         15, 565, 250, 270,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(action_label), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(action_label), 0.0, 0.5);

  point_label = gtk_label_new("");
  gtk_widget_show(point_label);
  gtk_table_attach(GTK_TABLE(table), point_label,
         15, 565, 270, 290,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(point_label), red_text_style);
  gtk_misc_set_alignment(GTK_MISC(point_label), 0.0, 0.5);

  path_label = gtk_label_new("");
  gtk_widget_show(path_label);
  gtk_table_attach(GTK_TABLE(table), path_label,
         15, 565, 290, 310,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_misc_set_alignment(GTK_MISC(path_label), 0.0, 0.5);

  file_label = gtk_label_new("");
  gtk_widget_show(file_label);
  gtk_table_attach(GTK_TABLE(table), file_label,
         15, 565, 310, 330,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_misc_set_alignment(GTK_MISC(file_label), 0.0, 0.5);

  status_label = gtk_label_new("");
  gtk_widget_show(status_label);
  gtk_table_attach(GTK_TABLE(table), status_label,
         15, 565, 330, 350,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(status_label), blue_text_style);
  gtk_misc_set_alignment(GTK_MISC(status_label), 0.0, 0.5);

  status_extra_label = gtk_label_new("");
  gtk_widget_show(status_extra_label);
  gtk_table_attach(GTK_TABLE(table), status_extra_label,
       15, 565, 350, 370,
       // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions         
       0, 0);  // xpadding, ypadding
  gtk_widget_set_style(GTK_WIDGET(status_extra_label), purple_text_style);
  gtk_misc_set_alignment(GTK_MISC(status_extra_label), 0.0, 0.5);
}



void Frontend::fill_backup_options_page(GtkWidget *vbox)
{
  GtkWidget *frame, *wid, *table, *table2, *sw;

  table = gtk_table_new(380, 580, TRUE); // 380 rows, 580 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(vbox), table);
  gtk_widget_show (table);

  frame = gtk_frame_new("Compression");
  gtk_widget_show (frame);

  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);

  gtk_table_attach(GTK_TABLE(table), frame,
       10, 120, 10, 170, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  table2 = gtk_table_new(130, 110, TRUE); // 140 rows, 110 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  none_radio = gtk_radio_button_new_with_label(compression_group, "NONE");
  gtk_widget_show (none_radio);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), none_radio,
      "NONE\n"
      "If selected, files will be saved with no compression.\n"
      "This can occupy a lot of space!",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), none_radio,
         10, 100, 5, 35, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  compression_group = gtk_radio_button_group(GTK_RADIO_BUTTON(none_radio));
  gz_radio = gtk_radio_button_new_with_label(compression_group, "GZIP");
  gtk_widget_show (gz_radio);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), gz_radio,
      "GZIP\n"
      "If selected, files will be saved with gzip compression.\n"
      "This seems to be the best average choice.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), gz_radio,
         10, 100, 40, 70, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  compression_group = gtk_radio_button_group(GTK_RADIO_BUTTON(gz_radio));
  bz2_radio = gtk_radio_button_new_with_label(compression_group, "BZIP2");
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), bz2_radio,
      "BZIP2\n"
      "If selected, files will be saved with bzip2 compression.\n"
      "This seems to be the best choice for larger files.",
      NULL);
  gtk_widget_show (bz2_radio);
  gtk_table_attach(GTK_TABLE(table2), bz2_radio,
         10, 100, 75, 105, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  compression_group = gtk_radio_button_group(GTK_RADIO_BUTTON(gz_radio));
  gz_bz2_radio = gtk_radio_button_new_with_label(compression_group, "GZ+BZ2");
  gtk_widget_show (gz_bz2_radio);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), gz_bz2_radio,
      "GZ+BZ2\n"
      "If selected, files will be saved with gzip and bzip2 compression.\n"
      "This seems to be of doubtful use especially for smaller files!.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), gz_bz2_radio,
         10, 100, 110, 140, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  if (spiffa->compression == "GZ")
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gz_radio), TRUE);
    }
  else if (spiffa->compression == "BZ2")
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bz2_radio), TRUE);
    }
  else if (spiffa->compression == "GZ_BZ2")
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gz_bz2_radio), TRUE);
    }
  else
    {
    // default is no compression
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(none_radio), TRUE);
    }

  frame = gtk_frame_new("Old Backups");
  gtk_widget_show (frame);
  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);
  gtk_table_attach(GTK_TABLE(table), frame,
       10, 120, 190, 270, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  table2 = gtk_table_new(100, 110, TRUE); // 100 rows, 110 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  wid = gtk_label_new("Keep:"); 
  gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
  gtk_widget_show(wid);

  gtk_table_attach(GTK_TABLE(table2), wid,
         10, 100, 0, 20, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  keep_spinner = create_spinner(spiffa->keep_number, -1, 100);
  gtk_widget_show(keep_spinner);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), keep_spinner,
      "KEEP OLD BACKUPS\n"
      "Keep the specified number of backups\n"
      "in addition to the current one.\n"
      "If -1, all backups are retained.\n"
      "If 0, all old backups are deleted\n"
      "in the destination directory.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), keep_spinner,
         30, 80, 25, 50, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  sw = get_list_store(spiffa->sources, "Sources to backup");
  gtk_table_attach(GTK_TABLE(table), sw,
       130, 310, 10, 375, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  frame = gtk_frame_new("New Source");
  gtk_widget_show (frame);
  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);
  gtk_table_attach(GTK_TABLE(table), frame,
       320, 570, 10, 115, // 140, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  table2 = gtk_table_new(120, 250, TRUE); // 160 rows, 250 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  new_source_entry = gtk_entry_new();
  gtk_widget_show (new_source_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), new_source_entry,
      "NEW SOURCE\n"
      "Enter a source for backup.\n"
      "NOTE: The source must be its full pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), new_source_entry,
         5, 240, 0, 40, // 10, 50
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("ADD New Source");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_add_new_source), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "ADD NEW SOURCE\n"
      "Add the selected source to the list of sources.\n"
      "NOTE: The source must be its full pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), wid,
       5, 145, 40, 80, // 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
       
  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_new_source), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for NEW SOURCE\n"
      "Select a source for backup.\n"
      "NOTE: The source must be its full pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), wid,
       150, 240, 40, 80, // 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("DELETE Selected Source");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_delete_source), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "DELETE SELECTED SOURCE\n"
      "Click on a source in the list to highlight it\n"
      "then clicking this button will delete it from the list.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       320, 570, 120, 160, // 175, 215, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  frame = gtk_frame_new("Mount Point");
  gtk_widget_show (frame);
  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);
  gtk_table_attach(GTK_TABLE(table), frame,
       320, 570, 162, 267,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
  table2 = gtk_table_new(120, 250, TRUE); // 120 rows, 250 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  mount_point_entry = gtk_entry_new();
  gtk_widget_show (mount_point_entry);
  gtk_entry_set_text(GTK_ENTRY(mount_point_entry),
                    spiffa->mount_point.c_str());
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), mount_point_entry,
      "MOUNT POINT\n"
      "Enter the mount point of the backup device\n"
      "where the backup will be placed. This is optional.\n"
      "If the mount point directory is a persistent directory\n"
      "and the device is not mounted the backup would be written\n"
      "to that directory instead of the device. If a mount point\n"
      "is specified spiffa will perform a primitive check that\n"
      "the mount point is actually mounted.\n"
      "NOTE: This must be its full pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), mount_point_entry,
       10, 240, 0, 40, // 10, 50, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_mount_point), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for MOUNT POINT\n"
      "Select the mount point for the backup destination device.\n"
      "NOTE: This must be its full pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), wid,
       150, 240, 40, 80, // 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
 
  frame = gtk_frame_new("Destination Directory");
  gtk_widget_show (frame);
  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);
  gtk_table_attach(GTK_TABLE(table), frame,
       320, 570, 270, 375, // 245, 375, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  table2 = gtk_table_new(120, 250, TRUE); // 120 rows, 250 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  backup_destination_entry = gtk_entry_new();
  gtk_widget_show (backup_destination_entry);
  gtk_entry_set_text(GTK_ENTRY(backup_destination_entry),
                    spiffa->backup_destination.c_str());
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), backup_destination_entry,
      "DESTINATION DIRECTORY\n"
      "Enter the destination directory where\n"
      "the backup will be placed.\n"
      "NOTE: This must be its full pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), backup_destination_entry,
       10, 240, 0, 40, // 10, 50, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_destination), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for DESTINATION DIRECTORY\n"
      "Select the destination directory where\n"
      "the backup will be placed.\n"
      "NOTE: This must be its full pathname.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), wid,
       150, 240, 40, 80, // 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
 
  wid = gtk_button_new_with_label("SAVE\nTHIS\nPAGE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_save_backup_data), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "SAVE THIS PAGE\n"
      "Saves the data from this page to \""
      SYSCONFDIR
      "/spiffa.conf\"\n",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       10, 120, 295, 375, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

}

void Frontend::fill_other_options_page(GtkWidget *vbox)
{
  GtkWidget *wid, *frame, *table, *table2, *sw;

  //  cout << "Frontend::fill_backup_omissions_page\n";

  table = gtk_table_new(380, 580, TRUE); // 380 rows, 580 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(vbox), table);
  gtk_widget_show (table);

  frame = gtk_frame_new("spiffa.log");
  gtk_widget_show (frame);

  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);

  gtk_table_attach(GTK_TABLE(table), frame,
       10, 120, 10, 140, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  table2 = gtk_table_new(130, 110, TRUE); // 140 rows, 110 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  new_log = gtk_radio_button_new_with_label(log_group, "NEW");
  gtk_widget_show (new_log);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), new_log,
      "NEW\n"
      "Select this if \""
      LOGDIR
      "/spiffa.log\"\n"
      "should be overwritten every backup, update or restore.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), new_log,
         10, 100, 5, 35, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  log_group = gtk_radio_button_group(GTK_RADIO_BUTTON(new_log));
  append_log = gtk_radio_button_new_with_label(log_group, "APPEND");
  gtk_widget_show (append_log);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), append_log,
      "APPEND\n"
      "Select this if \""
      LOGDIR
      "/spiffa/spiffa.log\"\n"
      "should have data appended every backup, update or restore.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), append_log,
         10, 100, 40, 70, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  log_group = gtk_radio_button_group(GTK_RADIO_BUTTON(append_log));
  no_log = gtk_radio_button_new_with_label(log_group, "NONE");
  gtk_widget_show (no_log);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), no_log,
      "NONE\n"
      "Select this if \""
      LOGDIR
      "/spiffa.log\"\n"
      "is not required.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), no_log,
         10, 100, 75, 105, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  if (spiffa->log_type == "NEW")
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(new_log), TRUE);
    }
  else if (spiffa->log_type == "APPEND")
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(append_log), TRUE);
    }
  else
    {
    // default is no log
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(no_log), TRUE);
    }

  sw = get_list_store(spiffa->omissions, "Sources to omit");
  gtk_table_attach(GTK_TABLE(table), sw,
       130, 310, 10, 375,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  frame = gtk_frame_new("New omission");
  gtk_widget_show (frame);
  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);
  gtk_table_attach(GTK_TABLE(table), frame,
       320, 570, 10, 140, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  table2 = gtk_table_new(120, 250, TRUE); // 120 rows, 250 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  new_omission_entry = gtk_entry_new();
  gtk_widget_show (new_omission_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), new_omission_entry,
      "NEW OMISSION\n"
      "Enter the full pathname of a file or directory\n"
      "that is NOT for backup.\n"
      "NOTE: Omissions take precedence over sources for backup.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), new_omission_entry,
         5, 240, 10, 50, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("ADD New omission");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
       G_CALLBACK(callback_add_new_omission), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "ADD NEW OMISSION\n"
      "Add the selected omission to the list of omissions.\n"
      "NOTE: The omission must be its full pathname.\n"
      "Omissions take precedence over sources for backup.\n"
      "Directories \"/proc\"  and  \"/sys\" are always omitted.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), wid,
       5, 145, 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_new_omission), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for NEW OMISSION\n"
      "Select the full pathname of a file or directory\n"
      "that is NOT for backup.\n"
      "NOTE: Omissions take precedence over sources for backup.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), wid,
       150, 240, 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("DELETE Selected Omission");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
       G_CALLBACK(callback_delete_omission), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "DELETE SELECTED OMISSION\n"
      "Click on an omission in the list to highlight it\n"
      "then clicking this button will delete it from the list.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       320, 570, 175, 215,  
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND |GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  save_files_check_button = gtk_check_button_new_with_label("Save existing files if different");
  gtk_widget_show (save_files_check_button);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), save_files_check_button,
      "SAVE EXISTING FILES\n"
      "When restoring files, an existing file is normally\n"
      "replaced by the file that is being restored.  If this\n"
      "is selected then, if different, pre-existing files will\n"
      " be saved with a suffix, \"filenameXXX.spiffasave\",\n"
      " where filename is the original filename and XXX is\n"
      "a decimal number.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), save_files_check_button,
       320, 570, 240, 270, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(save_files_check_button), spiffa->save_existing_files);

  restore_orphan_files_check_button = gtk_check_button_new_with_label("Restore orphan files on restore");
  gtk_widget_show (restore_orphan_files_check_button);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), restore_orphan_files_check_button,
      "RESTORE ORPHAN FILES\n"
      "Orphan files are files which have been deleted in the time\n"
      "between a backup and an update (incremental) backup.  Select\n"
      "this to restore the orphans as well as everything else.",
      NULL);
  gtk_table_attach(GTK_TABLE(table),restore_orphan_files_check_button,
       320, 570, 280, 310, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(restore_orphan_files_check_button), spiffa->restore_orphans);

  wid = gtk_button_new_with_label("SAVE\nTHIS\nPAGE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_save_other_data), this);
      
   gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "SAVE THIS PAGE\n"
      "Saves the data from this page to \""
      SYSCONFDIR
      "/spiffa.conf\"",
      NULL);
     
      
  gtk_table_attach(GTK_TABLE(table), wid,
       10, 120, 295, 375,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
}



void Frontend::fill_users_page(GtkWidget *vbox)
{
  GtkWidget *wid, *frame, *table, *table2, *sw;

  // cout << "Frontend::fill_users_page\n";

 table = gtk_table_new(380, 580, TRUE); // 380 rows, 580 columns, homogeneous
 gtk_container_add(GTK_CONTAINER(vbox), table);
 gtk_widget_show (table);

 sw = get_list_store(spiffa->users, "Users to allow");

 gtk_table_attach(GTK_TABLE(table), sw,
       130, 310, 10, 375, 
      // left attach, right attach, top attach, bottom attach
      (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
      // xoptions
      (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
      // yoptions
      0, 0);   // xpadding, ypadding

  frame = gtk_frame_new("New User");
  gtk_widget_show (frame);
  gtk_frame_set_label_align(GTK_FRAME(frame), .5, .5);
  gtk_table_attach(GTK_TABLE(table), frame,
       320, 570, 10, 140, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  table2 = gtk_table_new(120, 250, TRUE); // 160 rows, 250 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(frame), table2);
  gtk_widget_show (table2);

  new_user_entry = gtk_entry_new();
  gtk_widget_show (new_user_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), new_user_entry,
      "NEW USER\n"
      "Enter a  user name.\n"
      "NOTE: This the login name of the user.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), new_user_entry,
         5, 240, 10, 50, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("ADD New User");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
       G_CALLBACK(callback_add_new_user), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "ADD NEW USER\n"
      "Add this user to the list of users.\n"
      "NOTE: This the login name of the user.",
      NULL);
  gtk_table_attach(GTK_TABLE(table2), wid,
       5, 145, 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("DELETE Selected User");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
       G_CALLBACK(callback_delete_user), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "DELETE SELECTED USER\n"
      "Click on a user in the list to highlight it\n"
      "then clicking this button will delete it from the list.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       320, 570, 175, 215, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("SAVE\nTHIS\nPAGE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_save_user_data), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "SAVE THIS PAGE\n"
      "Saves the data from this page to \""
      SYSCONFDIR
      "/spiffa.conf\"",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       10, 120, 295, 375, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
}


void Frontend::fill_pathnames_page(GtkWidget *vbox)
{
  GtkWidget *wid, *table;

  //  cout << "Frontend::fill_pathnames_page\n";

  table = gtk_table_new(380, 580, TRUE); // 380 rows, 580 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(vbox), table);
  gtk_widget_show (table);


  wid = gtk_label_new("sh");
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       130, 170, 10, 50, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  sh_pn_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(sh_pn_entry),
                    spiffa->sh_pname.c_str());
  gtk_widget_show (sh_pn_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), sh_pn_entry,
      "SH\n"
      "Enter the full pathname of \"sh\" which invokes commands.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), sh_pn_entry,
         180, 480, 10, 50, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_sh_pn), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for SH\n"
      "Select the full pathname of \"sh\" which invokes commands.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 10, 50, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_label_new("cat");
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       130, 170, 52, 92, // 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  cat_pn_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(cat_pn_entry),
                    spiffa->cat_pname.c_str());
  gtk_widget_show (cat_pn_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), cat_pn_entry,
      "CAT\n"
      "Enter the full pathname of \"cat\" which concatenates files.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), cat_pn_entry,
         180, 480, 52, 92, // 60, 100,
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_cat_pn), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for CAT\n"
      "Select the full pathname of \"cat\" which concatenates files.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 52, 92, // 60, 100, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_label_new("gzip");
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       130, 170, 94, 134, // 110, 150, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  gzip_pn_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(gzip_pn_entry),
                    spiffa->gzip_pname.c_str());
  gtk_widget_show (gzip_pn_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), gzip_pn_entry,
      "GZIP\n"
      "Enter the full pathname of \"gzip\" which compresses files.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), gzip_pn_entry,
         180, 480, 94, 134, // 110, 150, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_gzip_pn), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for GZIP\n"
      "Select the full pathname of \"gzip\" which compresses files.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 94, 134, // 110, 150, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
 
  wid = gtk_label_new("bzip2");
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       130, 170, 136, 176, // 160, 200, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  bzip2_pn_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(bzip2_pn_entry),
                    spiffa->bzip2_pname.c_str());
  gtk_widget_show (bzip2_pn_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), bzip2_pn_entry,
      "BZIP2\n"
      "Enter the full pathname of \"bzip2\" which compresses files.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), bzip2_pn_entry,
         180, 480, 136, 176, // 160, 200, 
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_bzip2_pn), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for BZIP2\n"
      "Select the full pathname of \"bzip2\" which compresses files.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 136, 176, // 160, 200, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_label_new("ln");
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       130, 170, 178, 218, // 210, 250,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  ln_pn_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(ln_pn_entry),
                    spiffa->ln_pname.c_str());
  gtk_widget_show (ln_pn_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), ln_pn_entry,
      "LN\n"
      "Enter the full pathname of \"ln\" which creates links.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), ln_pn_entry,
         180, 480, 178, 218, // 210, 250,
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_ln_pn), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for LN\n"
      "Select the full pathname of \"ln\" which creates links.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 178, 218, // 210, 250,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_label_new("mtab");
  gtk_misc_set_alignment(GTK_MISC(wid), 0.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       130, 170, 220, 260,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  mtab_pn_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(mtab_pn_entry),
                    spiffa->mtab_pname.c_str());
  gtk_widget_show (mtab_pn_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), mtab_pn_entry,
      "MTABn"
      "Enter the full pathname of \"mtab\" which stores mounted file system data.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), mtab_pn_entry,
         180, 480, 220, 260,
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("BROWSE");
  gtk_widget_show (wid);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_mtab_pn), this);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "BROWSE for MTAB\n"
      "Select the full pathname of \"mtab\" which stores mounted file system data.",
      NULL);
  gtk_table_attach(GTK_TABLE(table), wid,
       490, 570, 220, 260,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_label_new("Browser Command");
  gtk_misc_set_alignment(GTK_MISC(wid), 1.0, 0.5);
  gtk_widget_show (wid);
  gtk_table_attach(GTK_TABLE(table), wid,
       10, 170, 262, 302, // 260, 300,
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
  browser_pn_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(browser_pn_entry),
         spiffa->browser_pname.c_str());
  gtk_widget_show (browser_pn_entry);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), browser_pn_entry,
      "BROWSER COMMAND\n"
      "Enter the full pathname with argument for the browser\n"
      "to display a file e.g. \"htmlview %s\" where\"%s\"\n"
      "represents the pathname of the file. This is used\n"
      "to display \"Help\". If the file is not at the expected\n"
      "place, enter it's full pathname instead of \"%s\".",
      NULL);
  gtk_table_attach(GTK_TABLE(table), browser_pn_entry,
         180, 480, 262, 302, // 260, 300,
         // left attach, right attach, top attach, bottom attach
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
         (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
         0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("INSERT DEFAUTS");
  gtk_widget_show (wid);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "INSERT DEFAULTS\n"
      "Places default data into the entry widgets.",
      NULL);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_pathnames_defaults), this);
  gtk_table_attach(GTK_TABLE(table), wid,
       180, 480, 310, 375, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  wid = gtk_button_new_with_label("SAVE\nTHIS\nPAGE");
  gtk_widget_show (wid);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "SAVE THIS PAGE\n"
      "Saves the data from this page to \""
      SYSCONFDIR
      "/spiffa.conf\"",
      NULL);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_save_pathnames), this);
  gtk_table_attach(GTK_TABLE(table), wid,
       10, 120, 295, 375, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
}

void Frontend::fill_about_page(GtkWidget *vbox)
{

  GtkWidget *scrolled_window, *table;
  GtkWidget *wid;
  GtkWidget *box;

  string message = version_txt;
  message += " : ";
  message += copyright_txt;
  message += "\n";
  message +=  email_contact_txt;
  message += "\n";
  message += snail_contact_txt;
  message +=   "\n\n";
  message +=  
"    This program is free software; you can redistribute it and/or modify\n"
"    it under the terms of the GNU General Public License as published by\n"
"    the Free Software Foundation; either version 2 of the License or,\n"
"    at your option, any later version.\n\n"

"    This program is distributed in the hope that it will be useful,\n"
"    but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. \n"
"    See the GNU General Public License for more details.\n\n"

"    You should have received a copy of the GNU General Public License\n"
"    along with this program; if not, write to the Free Software\n"
"    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n";
    
  table = gtk_table_new(380, 580, TRUE); // 380 rows, 580 columns, homogeneous
  gtk_container_add(GTK_CONTAINER(vbox), table);
  gtk_widget_show (table);

  scrolled_window = gtk_scrolled_window_new(NULL, NULL);

    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
  gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolled_window),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
  gtk_widget_show (scrolled_window);

  gtk_table_attach(GTK_TABLE(table), scrolled_window,
       10, 570, 10, 340, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding

  box = gtk_vbox_new(FALSE, 0);
  gtk_widget_show (box);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
          box);

  wid = gtk_label_new(message.c_str());
  gtk_widget_show (wid);
  gtk_container_add(GTK_CONTAINER(box), wid);


  wid = gtk_button_new_with_label("USER MANUAL");
  gtk_widget_show (wid);
  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), wid,
      "USER MANUAL\n"
      "Browse the documentation",
      NULL);
  g_signal_connect(G_OBJECT (wid), "clicked",
          G_CALLBACK(callback_browse_docs), this);

  gtk_table_attach(GTK_TABLE(table), wid,
       225, 350, 340, 380, 
       // left attach, right attach, top attach, bottom attach
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // xoptions
       (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
       // yoptions
       0, 0);   // xpadding, ypadding
}




// Backup via spiffaslave
void Frontend::callback_backup(GtkWidget *widget, Frontend *f)
{
  //  cout << "Frontend::callback_backup\n";

  if (f->timer)
    {
      f->report("perform backup", f);
    }
  else
    {
      string file = gtk_entry_get_text(GTK_ENTRY(f->config_entry));

      // remove junk
      remove_leading_and_terminal_space(file);

      string action = "New Backup - ";
      if (f->spiffa->compression == "GZ")
        action += "gzip";
      else if (f->spiffa->compression == "BZ2")
        action += "bzip2";
      else if  (f->spiffa->compression == "GZ_BZ2")
        action += "gzip + bzip2";
      else action += "no";
      
      action += " compression : using ";
      if (file.length())
        {
        action += "\"";
        action += file;
        action += "\"";
        }
      else action += "default configuration";
      f->clear_labels(f);
      gtk_label_set(GTK_LABEL(f->action_label), action.c_str());

      f->rainbow_world_image = f->rainbow_world_image_bu;
      f->spiffa->slave_working = TRUE;
      f->action = F_BACKUP;
      f->m->send_message(SP_BACKUP, (char *)file.c_str());
      f->start_timer(f);
    }
}

// Update via spiffaslave
void Frontend::callback_update(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_update\n";
if (f->timer)
  {
  f->report("perform update", f);
  }
else
  {
  string file = gtk_entry_get_text(GTK_ENTRY(f->config_entry));

  // remove junk
  remove_leading_and_terminal_space(file);

  string action =  "Update Most Recent Backup : using ";
  if (file.length())
    {
    action += "\"";
    action += file;
    action += "\"";
    }
  else action += "default configuration";
  f->clear_labels(f);
  gtk_label_set(GTK_LABEL(f->action_label), action.c_str());

  f->rainbow_world_image = f->rainbow_world_image_bu;
  f->spiffa->slave_working = TRUE;
  f->action = F_UPDATE;
  f->m->send_message(SP_UPDATE, (char *)file.c_str());
  f->start_timer(f);
  }
}


void Frontend::callback_restore_from(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_restore_from\n";

string from = gtk_entry_get_text(GTK_ENTRY(f->from_entry));
string file = gtk_entry_get_text(GTK_ENTRY(f->config_entry));
string text;

if (f->timer)
    {
    f->report("perform restore", f);
    }
else if (f->restore_selection.empty())
    {
    text = "Restore from: ";

    text += (from.length()) ? "selected restore point" : "latest backup";
    f->clear_labels(f);

    gtk_label_set(GTK_LABEL(f->action_label), text.c_str());


    f->rainbow_world_image = f->rainbow_world_image_rs;
    f->spiffa->slave_working = TRUE;
    f->action = F_RESTORE;

    text = from + "\n";
    text += file;
    f->m->send_message(SP_RESTORE_FROM, (char *)text.c_str());
    f->start_timer(f);
    }
else // Restoring a selection
    {
    list<string>::iterator i;
        
    f->clear_labels(f);
    gtk_label_set(GTK_LABEL(f->action_label), "Restore from selection");

    f->rainbow_world_image = f->rainbow_world_image_rs;
    f->spiffa->slave_working = TRUE;
    f->action = F_RESTORE;

    // The selection has to be passed to our slave
    f->m->send_message(SP_START_RESTORE_SELECTION, NULL);
    for (i=f->restore_selection.begin(); i!=f->restore_selection.end(); ++i)
        {
        f->m->send_message(SP_RESTORE_SELECTION_NAME, (char *)(*i).c_str());
        }
    text = from + "\n";
    text += file;
    f->m->send_message(SP_END_RESTORE_SELECTION, (char *)text.c_str());
    f->start_timer(f);
    }
}



void Frontend::close_file_requester(GtkWidget *widget,  Frontend *f)
{
//cout << "Frontend::close_file_requester\n";

if (f->filew)
  {
  GtkWidget *fw = f->filew;
  f->filew = NULL;

  gtk_widget_destroy(GTK_WIDGET(fw));
  }
}

// Return the default restore point in restore_point
bool Frontend::get_default_restore_point(string *backup_destination, string *restore_point)
{
DIR *dp;
struct dirent *de;
string s_name;
string msg;
    
double stamp = 0;
double test_stamp;
string hostname;
char buffer[128]; // unnecessarily large!
char *b;          // position in buffer
char *p;          // position in current name being checked
string basename;

// cout << "Restore::find_backup\n";

find_hostname(&hostname, &msg);
if (msg.length())
    {
    cerr << msg << endl;
    *restore_point = "";
    return FALSE;
    }
    
basename = "spiffa_backup_";
basename += hostname;
basename += "_";

dp = opendir(backup_destination->c_str());
if (dp)
    {
    while ( (de = readdir(dp)) )
        {
        //cout <<  "Restore::find_backup - got1 " << de->d_name << endl;

        if ( STREQN(de->d_name, (char *)basename.c_str(), basename.length() ) )
            {
     
            // Get its datestamp from its name (YYYY_MM_DD_hh_mm_ss)
            p = &de->d_name[basename.length()]; // skip over the basename

            b = buffer;
 
            if (strlen(p) != 19)
                {
                // BAD NAME
                // cout <<  "Restore::find_backup - BAD NAME1 " << p << endl;
                continue;
                }
            while (*p)
                {
                if (*p == '_') {;} // skip a '_'
                else if ( (*p) && isdigit(*p) ) *b++ = *p;
                else
                    {
                    // BAD NAME
                    // cout <<  "Restore::find_backup - BAD NAME2 " << p << endl;
                    break;
                    }
                p++;
                }
            *b = 0;
            
            if ((strlen(buffer) != 14) || *p)
                {
                // BAD NAME
                // cout <<  "Restore::find_backup - BAD NAME3 " << p << endl;
                continue;
                }

            test_stamp = atof(buffer);
          
            if (test_stamp > stamp)
                {
                 
                stamp = test_stamp;
                s_name = *backup_destination;
                 if ( (*backup_destination)[ backup_destination->length()-1 ] != '/')
                    {
                    s_name += '/';
                    }
                s_name += de->d_name;
                }
            }
        } // End of while readdir
    closedir(dp);
    }
    
if (s_name.length())
    {
    *restore_point = s_name;
    return TRUE;
    }
else *restore_point = "";
return FALSE;
}


// The restore point pathname of a "from" filename
bool Frontend::get_restore_point_pathname(string *from, string *pathname)
{
  string basename;
  string hostname;
  string name;
  unsigned int n;
  string msg;
  
  find_hostname(&hostname, &msg);
  if (msg.length())
    {
    // ERROR   
    cerr << msg << endl;
    *pathname = "";
    return FALSE;
    }

  basename = "spiffa_backup_";
  basename += hostname;
  basename += "_";

  for (n=0; n<from->length(); ++n)
    {
      if ( STREQN( (char *)(from->substr(n)).c_str(),
       (char *)basename.c_str(),
       basename.length() ) )
        {
        unsigned int ln = 0;

        string directory = from->substr(0, n);

        while ( (n + ln) <= from->length() )
            {
            if ( ( (*from)[n+ln] == '/' ) || ( (n+ln) == from->length() ) )
                {
                *pathname = directory + name;
                return ( ((ln - basename.length()) == 19) ? TRUE : FALSE );
                }
            name += (*from)[n+ln];
            ln++;
            }
        break;
        }
    }

  return FALSE;
}


void Frontend::insert_selected(GtkWidget *widget,  Frontend *f)
{
string pathname = (char *)gtk_file_selection_get_filename (GTK_FILE_SELECTION (f->filew));


// remove junk
remove_leading_and_terminal_space(pathname);

// This has to be put into the correct entry widget
if (f->entry_for_select == NULL)  // marker for saving all data to pathname
  {
  //      cout << "Frontend::insert_selected - got config pathname " << pathname << endl;
  if (f->timer)
    {
    f->report((char *)( (f->action == F_LOAD) ? "load selected configuration data":
                                 "\"save as\" configuration data"),
              f);
    }
  else if (action == F_LOAD)
    {
    if (pathname.length())
      {
      string action_msg = "Load configuration file \"";
      action_msg += pathname;
      action_msg += "\" for edit";

      // cout << "Frontend::insert_selected - " << action_msg << endl;

      f->clear_labels(f);
      gtk_label_set(GTK_LABEL(f->action_label), action_msg.c_str());

      // Where the image is displayed
      f->rainbow_world_image = f->rainbow_world_image_bu;
      f->spiffa->slave_working = TRUE;
      f->m->send_message(SP_LOAD, (char *)pathname.c_str());
      f->start_timer(f);
      }
    else
      {
      gtk_label_set(GTK_LABEL(f->gui_info_label), "Can't load: empty filename");
      }
    }
  else
    {
    string action_msg = "Saving configuration file as \"";
    action_msg += pathname;
    action_msg += "\"";

    // cout << "Frontend::insert_selected - " << action_msg << endl;

    f->clear_labels(f);

    // Where the image is displayed
    f->rainbow_world_image = f->rainbow_world_image_bu;

    f->spiffa->slave_working = TRUE;

    f->save_user_data(f);
    f->save_backup_data(f);
    f->save_other_data(f);
    f->save_pathnames(f);

    // label set has to come here because what goes before clears it.
    gtk_label_set(GTK_LABEL(f->action_label), action_msg.c_str());

    f->m->send_message(SP_CONF_DONE, (char *)pathname.c_str());
    f->start_timer(f);
    }
  }

else if (action == F_RESTORE_POINT)
   {  
    string restore_point;
    string current_restore_point;
    
 //   cout << "Frontend::insert_selected -    action == F_RESTORE_POINT\n" ;
    
    while ( (pathname.length()>1) && (pathname[pathname.length()-1]=='/') )
      {
      pathname = pathname.substr(0, (pathname.length()-1) );
       }
       
    //  Now extract the restore point (a sub-directory may have been chosen)  
    //  ================================================
    if (!(f->get_restore_point_pathname(&pathname, &restore_point)))
        {
        // ERROR
        string msg = "Can't obtain restore point from \"" + pathname;
        msg += "\"";
        gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
        restore_point = "";
        }
    current_restore_point = gtk_entry_get_text(GTK_ENTRY(f->entry_for_select));
    if ( (!f->restore_selection.empty()) && (current_restore_point != restore_point) )
        {
        f->restore_selection.clear();
        }
    gtk_entry_set_text(GTK_ENTRY(f->entry_for_select), restore_point.c_str());
    }

else // f->entry_for_select NOT NULL
  {
  // a file pathname
  // directory pathname
  // file OR directory pathname
  if (action==F_FILE_PATHNAME)
    {
    while ( (pathname.length()) && (pathname[pathname.length()-1]=='/') )
      {
      pathname = pathname.substr(0, (pathname.length()-1) );
      }
    }
  else if ((action==F_DIRECTORY_PATHNAME)||(action==F_FILE_OR_DIRECTORY_PATHNAME))
    {
    while ( (pathname.length()>1) && (pathname[pathname.length()-1]=='/') )
      {
      pathname = pathname.substr(0, (pathname.length()-1) );
      }
    }

  gtk_entry_set_text(GTK_ENTRY(f->entry_for_select), pathname.c_str());
  }

f->close_file_requester(widget, f);
}



void Frontend::get_a_name(GtkWidget *widget, Frontend *f, char *title, char *selection, bool dir_only)
{
//cout << "Frontend::get_a_name - selection= " << selection << endl;

if (!f->filew)
  {
  // Create a new file selection widget
  f->filew = gtk_file_selection_new (title);
    
  // Prevent interaction with other windows
  gtk_window_set_modal(GTK_WINDOW(f->filew), TRUE);

  if (f->spiffa_icon_pixbuf)
    {
    gtk_window_set_icon( GTK_WINDOW(f->filew), f->spiffa_icon_pixbuf);
    }

  if (dir_only)
    {
    gtk_widget_hide(GTK_FILE_SELECTION(f->filew)->file_list->parent);
    gtk_widget_hide(GTK_FILE_SELECTION(f->filew)->selection_entry);
    }

  if (selection) gtk_file_selection_set_filename(GTK_FILE_SELECTION(f->filew), selection);

  //  Connect the ok_button to file_ok_sel function
  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (f->filew)->ok_button),
      "clicked", G_CALLBACK (insert_selected), (gpointer) f);

  gtk_button_set_label(GTK_BUTTON( GTK_FILE_SELECTION(f->filew)->ok_button), "Select");

  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (f->filew)->cancel_button),
      "clicked", G_CALLBACK (f->close_file_requester), (gpointer) f);

  g_signal_connect(G_OBJECT (GTK_FILE_SELECTION (f->filew)),
           "destroy",
           G_CALLBACK (f->close_file_requester),
           (gpointer)f );

  // Hide unwanted buttons
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(f->filew));
    
  gtk_widget_show(f->filew);
  }
else
  {
  // Shouldn't get here because the file requester blocks the main window's input
  Complain *c = new Complain;
  string msg = "Can't open file requester.\nFile requester is already in use";
  c->complain(c->CN_OK,
              "Spiffa",
              (char *)msg.c_str(),
              NULL,
              NULL,
              f);

  //      cout << "Frontend::get_a_name - " << msg << endl;
  }
}


void Frontend::quit_selection(GtkWidget *widget, Frontend *f)
{
//cout << "Frontend::quit_selection" << endl;
if (f->select_dialog)
    {
    gtk_grab_remove (f->select_dialog);
    f->select_dialog = NULL;
    }
}

void Frontend::selection_cancel(GtkWidget *widget, Frontend *f)
{
//cout << "Frontend::selection_cancel" << endl;
if (f->select_dialog)
    {
    gtk_widget_destroy(f->select_dialog);
    }
}


void Frontend::selection_exposed(GtkWidget *widget, GdkEvent *event, Frontend *f)
{
gtk_label_set(GTK_LABEL(((Frontend *)f)->gui_info_label), ""); 

//cout << "Frontend::selection_exposed" << endl;

gtk_signal_disconnect(GTK_OBJECT(widget), f->selection_exposed_signal);
gtk_widget_set_size_request (f->select_dialog, -1, 500);  // w, h
}


// Get the selection by traversing the tree
// Put it into a list which should first be cleared
// Make each entry the full pathname of that name in the tree
void Frontend::selection_ok(GtkWidget *widget, Frontend *f)
{
GtkTreeIter iter;
gchar *restore_point;

gtk_label_get(GTK_LABEL(f->restore_selection_label), &restore_point);  // restore point

f->restore_selection.clear();

// Now traverse the tree to extract the info

if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(f->model), &iter))
    {
    gchar *tree_name;
    gboolean flag;
    
    // Get the first name - should be "root"
    
    gtk_tree_model_get (GTK_TREE_MODEL(f->model), &iter,
                      COL_NAME, &tree_name,
                      COL_CHECKBUTTON, &flag,
                      -1);

    if (STREQ( (char *)tree_name, "root" ))
        {
        string parent_name = (char *)restore_point;
        parent_name +=  "/";
        parent_name += tree_name;
        g_free(tree_name);

        if (flag)
            {
            // Traverse this directory
            f->traverse_descendents(&iter, &parent_name, f);
            }
        }
    else
        {
        // ERROR
        gtk_label_set(GTK_LABEL(f->gui_info_label), "ERROR: Root of selection is not \"root\"!");
        g_free(tree_name);
        }
    }

// If there is a selection, the label "Restore Point" should be changed to "Restore Selection"
// Also maybe place restore point into its the edit
if (f->restore_selection.empty())
    {
    gtk_label_set(GTK_LABEL(f->restore_point_label), "Restore Point");
    gtk_widget_set_style(GTK_WIDGET(f->restore_point_label), f->black_text_style);
    }
else
    {
    gtk_label_set(GTK_LABEL(f->restore_point_label), "Restore Selection");
    gtk_widget_set_style(GTK_WIDGET(f->restore_point_label), f->red_text_style);
    }

f->selection_cancel(widget, f);
}


void Frontend::traverse_descendents(GtkTreeIter *parent, string *parent_name, Frontend *f)
{
GtkTreeIter child;

// Add the root to the selection
f->restore_selection.push_back(*parent_name);

// If this parent has any children
if (gtk_tree_model_iter_children(GTK_TREE_MODEL(f->model), &child, parent))
    {
    gchar *tree_name;
    gboolean flag;
    
    
    gtk_tree_model_get (GTK_TREE_MODEL(f->model), &child,
                      COL_NAME, &tree_name,
                      COL_CHECKBUTTON, &flag,
                      -1);
    if (flag)
        {
        string name = *parent_name;
        name += "/";
        name += tree_name;
        g_free(tree_name);
        f->traverse_descendents(&child, &name, f);
        }
    else g_free(tree_name);
    
    while (gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &child))
	    {
        gtk_tree_model_get (GTK_TREE_MODEL(f->model), &child,
                      COL_NAME, &tree_name,
                      COL_CHECKBUTTON, &flag,
                      -1);
        if (flag)
            {
            string name = *parent_name;
            name += "/";
            name += tree_name;
            g_free(tree_name);
            f->traverse_descendents(&child, &name, f);
            }
        else g_free(tree_name);
	    }
    }
}


// Set Check for all descendents of parent (descended from ancestor)
void Frontend::fix_descendents(GtkTreeIter *parent, gboolean mark_flag)
{
GtkTreeIter child;

// SET CHECK
gtk_tree_store_set(treestore, parent, COL_CHECKBUTTON, mark_flag, -1);

// If this parent has any children
if (gtk_tree_model_iter_children(GTK_TREE_MODEL(model), &child, parent))
    {
      fix_descendents(&child, mark_flag);
      while (gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &child))
	    {
	    fix_descendents( &child, mark_flag);
	    }
    }
}



// ===============================================================
// It should be possible to toggle the checkbutton here somehow.
// It doesn't happen automatically as one would have expected...
// ===============================================================
void Frontend::toggle_func(GtkCellRendererToggle *cell_renderer, gchar *path, Frontend *f)
{
GtkTreeIter iter;
GtkTreePath *tree_path;

//  g_print("toggle_func \n");

tree_path = gtk_tree_path_new_from_string(path);

if (gtk_tree_model_get_iter(GTK_TREE_MODEL(f->model), &iter, tree_path))
    {
    if (gtk_cell_renderer_toggle_get_active(GTK_CELL_RENDERER_TOGGLE(cell_renderer)))
	    {
	    GtkTreeIter child = iter;
	    
	    gtk_tree_store_set(f->treestore, &child, COL_CHECKBUTTON, FALSE, -1);
	    f->fix_descendents(&iter, FALSE);
        }
    else // Make it active
        {
	    GtkTreeIter parent;
	    
	    GtkTreeIter child = iter;
	            
	    // Now get the parents and make them TRUE as well
	    while (gtk_tree_model_iter_parent(GTK_TREE_MODEL(f->model), &parent, &child))
	        {
	        gtk_tree_store_set(f->treestore, &parent, COL_CHECKBUTTON, TRUE, -1);
	        if (!(gtk_tree_model_iter_parent(GTK_TREE_MODEL(f->model), &parent, &child))) break;
	        child = parent;
	        }
        f->fix_descendents(&iter, TRUE);
        }
    }

else cerr << "gtk_tree_model_get_iter FAILED\n";

gtk_tree_path_free (tree_path);
}


void Frontend::populate_directory(GtkTreeStore *treestore, GtkTreeIter *parent, string *dirpathname)
{
GtkTreeIter child;
DIR *dp;
struct dirent *de;
string pathname;


dp = opendir(dirpathname->c_str());
if (dp)
    {
    while ( (de = readdir(dp)) )
        {
        if ( STREQ(de->d_name, ".") || STREQ(de->d_name, "..")) continue;
 
        pathname = *dirpathname + "/";
        pathname += de->d_name;
        
        if (isdir((char *)pathname.c_str()))
            {
            GtkTreeIter dirchild ;
 
            gtk_tree_store_append(treestore, &dirchild, parent);  // It appears that parent is not a valid iter
            gtk_tree_store_set(treestore, &dirchild,
                                    COL_NAME, de->d_name,
                                    -1);
            // If this is in the restore selection list, make it selected
            if ( find(restore_selection.begin(), restore_selection.end(), pathname) != restore_selection.end())
                {
                gtk_tree_store_set(treestore, &dirchild, COL_CHECKBUTTON, TRUE, -1);
                }

            populate_directory(treestore, &dirchild, &pathname);
            }
        else
            {
            gtk_tree_store_append(treestore, &child, parent);
            gtk_tree_store_set(treestore, &child,
                                            COL_NAME, de->d_name,
                                            -1);
            // If this is in the restore selection list, make it selected
            if ( find(restore_selection.begin(), restore_selection.end(), pathname) != restore_selection.end())
                {
                gtk_tree_store_set(treestore, &child, COL_CHECKBUTTON, TRUE, -1);
                }

            }
        }
    closedir(dp);
    }
}

// Populate the tree
void Frontend::populate_tree(GtkTreeStore *treestore, string *restore_point)
{
GtkTreeIter  toplevel, child;

DIR *dp;
struct dirent *de;
string pathname;
string rootname;

rootname = *restore_point + "/root";
gtk_tree_store_append(treestore, &toplevel, NULL);
gtk_tree_store_set(treestore, &toplevel,
                     COL_NAME, "root",
                     -1);
                     
// If this is in the restore selection list, make it selected
if ( find(restore_selection.begin(), restore_selection.end(), rootname) != restore_selection.end())
    {
    gtk_tree_store_set(treestore, &toplevel, COL_CHECKBUTTON, TRUE, -1);
    }

dp = opendir(rootname.c_str());
if (dp)
    {
    while ( (de = readdir(dp)) )
        {
        if ( STREQ(de->d_name, ".") || STREQ(de->d_name, "..")) continue;
        pathname = rootname + "/";
        pathname += de->d_name;
        if (isdir((char *)pathname.c_str()))
            {
            GtkTreeIter dirchild ;
                        
            gtk_tree_store_append(treestore, &dirchild, &toplevel);
            gtk_tree_store_set(treestore, &dirchild,
                                            COL_NAME, de->d_name,
                                            -1);
            // If this is in the restore selection list, make it selected
            if ( find(restore_selection.begin(), restore_selection.end(), pathname) != restore_selection.end())
                {
                gtk_tree_store_set(treestore, &dirchild, COL_CHECKBUTTON, TRUE, -1);
                }

            populate_directory(treestore, &dirchild, &pathname);
            }
        else
            {
            gtk_tree_store_append(treestore, &child, &toplevel);
            gtk_tree_store_set(treestore, &child,
                                            COL_NAME, de->d_name,
                                            -1);
            // If this is in the restore selection list, make it selected
            if ( find(restore_selection.begin(), restore_selection.end(), pathname) != restore_selection.end())
                {
                gtk_tree_store_set(treestore, &child, COL_CHECKBUTTON, TRUE, -1);
                }
            }
        }
    closedir(dp);
    }
}



// ====================================================
// Here we have to populate the tree with the whole of the restore point's root
// ====================================================
GtkTreeModel *Frontend::create_and_fill_model(Frontend *f, string *restore_point)
{
GtkTreeIter    toplevel, child, another_child;

gtk_label_set(GTK_LABEL(f->restore_selection_label), restore_point->c_str());

treestore = gtk_tree_store_new(NUM_COLS,
                                                        G_TYPE_STRING,
                                                        G_TYPE_BOOLEAN);

// Now populate it, all directories and files string with "root"
// Should it be entered as "/" or "root" ?  - using "root"

populate_tree(treestore,  restore_point);

return GTK_TREE_MODEL(treestore);
}



GtkWidget *Frontend::create_view_and_model (Frontend *f, string *restore_point)
{
  GtkTreeViewColumn   *col;
  GtkCellRenderer     *renderer;
  GtkWidget           *view;
  view = gtk_tree_view_new();

  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);

  /* --- Column #1 --- */
  col = gtk_tree_view_column_new();

  gtk_tree_view_column_set_title(col, "Name");

  /* pack tree view column into tree view */
  gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
  renderer = gtk_cell_renderer_text_new();
  /* pack cell renderer into tree view column */
  gtk_tree_view_column_pack_start(col, renderer, TRUE);
  /* connect ~text~ property of the cell renderer to
   * model column that contains the first name */
  gtk_tree_view_column_add_attribute(col, renderer, "text", COL_NAME);


  /* --- Column #2 --- */
   renderer = gtk_cell_renderer_toggle_new ();
   col = gtk_tree_view_column_new_with_attributes ("Checkbutton",
                                                      renderer,
                                                      "active", COL_CHECKBUTTON,
                                                      NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (view), col);

    g_signal_connect (G_OBJECT(renderer), "toggled", G_CALLBACK(f->toggle_func), f);
  

    model = create_and_fill_model(f, restore_point);
    gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
    g_object_unref(model); /* destroy model automatically with view */
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
                              GTK_SELECTION_NONE);
    return view;
}


// Make selection from the restore point
// The restore point may be the default or a selected restore point
void Frontend::make_selection(Frontend *f)
{
GtkWidget *wid;
GtkWidget *view;
GtkWidget *sw;
string restore_point;
string pathname = gtk_entry_get_text(GTK_ENTRY(f->from_entry));

if (f->select_dialog) return;

if (pathname.length()==0)
    {
    if (!get_default_restore_point(&(f->spiffa->backup_destination), &restore_point))
        {
        gtk_label_set(GTK_LABEL(f->gui_info_label), "Failed to obtain default restore point");
        return;
        }
    }
else
    {
    // Fix up if it is a sub-directory...
    if (!(f->get_restore_point_pathname(&pathname, &restore_point)))
        {
        // ERROR
        string msg = "Can't obtain restore point from \"" + pathname;
        msg += "\"";
        gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
        return;
        }
    }

f->select_dialog = gtk_dialog_new ();
gtk_widget_show(f->select_dialog);

gtk_signal_connect (GTK_OBJECT (f->select_dialog), "destroy",
              GTK_SIGNAL_FUNC (f->quit_selection),
              this);
              
selection_exposed_signal = gtk_signal_connect (GTK_OBJECT (f->select_dialog), "expose_event",
             GTK_SIGNAL_FUNC (f->selection_exposed),
             this);
             
gtk_window_set_icon( GTK_WINDOW(f->select_dialog), spiffa_icon_pixbuf);
restore_selection_label = gtk_label_new("");
gtk_box_pack_start (GTK_BOX (GTK_DIALOG (f->select_dialog)->vbox), 
              restore_selection_label, FALSE, TRUE, 0);

/* --- Set the title and add a border --- */
gtk_window_set_title (GTK_WINDOW (f->select_dialog), "SELECT FOR RESTORE");
gtk_container_border_width (GTK_CONTAINER (f->select_dialog), 0);
gtk_window_set_modal(GTK_WINDOW(f->select_dialog), TRUE);


sw = gtk_scrolled_window_new (NULL, NULL);
gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
             GTK_SHADOW_ETCHED_IN);
gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
              GTK_POLICY_AUTOMATIC,
              GTK_POLICY_AUTOMATIC);
gtk_box_pack_start (GTK_BOX (GTK_DIALOG (f->select_dialog)->vbox), 
              sw, TRUE, TRUE, 0);


// This needs to be able to be scrolled
view = create_view_and_model(f, &restore_point);

gtk_container_add(GTK_CONTAINER(sw), view);

wid = gtk_button_new_with_label ("OK");
gtk_signal_connect (GTK_OBJECT (wid), "clicked",
        GTK_SIGNAL_FUNC (f->selection_ok),
        this);
 gtk_box_pack_start (GTK_BOX (GTK_DIALOG (f->select_dialog)->action_area), 
          wid, TRUE, TRUE, 0);
GTK_WIDGET_SET_FLAGS (wid, GTK_CAN_DEFAULT);
gtk_widget_show(wid);

wid = gtk_button_new_with_label ("CANCEL");
gtk_signal_connect (GTK_OBJECT (wid), "clicked",
        GTK_SIGNAL_FUNC (f->selection_cancel),
        this);
 gtk_box_pack_start (GTK_BOX (GTK_DIALOG (f->select_dialog)->action_area), 
          wid, TRUE, TRUE, 0);
GTK_WIDGET_SET_FLAGS (wid, GTK_CAN_DEFAULT);
gtk_widget_grab_default (wid);
gtk_widget_show(wid);

// Make sure the dialog keeps the focus
gtk_grab_add(f->select_dialog);
gtk_widget_grab_focus(f->select_dialog);
gtk_widget_show_all(f->select_dialog);
}



void Frontend::callback_browse_from(GtkWidget *widget, GdkEventButton *event, Frontend *f)
{
//cout << "Frontend::callback_browse_from\n";

f->clear_labels(f);
if  (event->type == GDK_BUTTON_PRESS)
    {
    switch (event->button)
        {
        case 1:     // LEFT BUTTON
            {
            // Needs a terminal slash if a directory
            string restore_point = f->spiffa->backup_destination;
            if (restore_point.length())
                {
                if (restore_point[restore_point.length()-1] != '/')
                    {
                    restore_point += '/';
                    }
                }
            else restore_point = "/";

            // Change this to F_RESTORE_POINT so that it can be checked on return when selected
            f->action = F_RESTORE_POINT;     // was F_DIRECTORY_PATHNAME; 

            f->entry_for_select = f->from_entry;
            f->get_a_name(widget, f, "Choose RESTORE POINT for Restore", (char *)restore_point.c_str(),  TRUE);
            }
        break;
        
        case 2:    // MIDDLE BUTTON
        case 3:    // RIGHT BUTTON
        default:
            {
//            cout << "Frontend::callback_browse_from - RIGHT BUTTON" << endl;

// This sort of works
           gtk_label_set(GTK_LABEL(((Frontend *)f)->gui_info_label), "Collecting selection data, please wait"); 
           gtk_widget_show_now(((Frontend *)f)->gui_info_label);
           gtk_main_iteration ();
             
            f->make_selection(f);
            }
        break;
        }
    }
}

void Frontend::callback_browse_config(GtkWidget *widget, Frontend *f)
{
unsigned int len;

string conf_dir = SYSCONFDIR;

if ( ( (len=conf_dir.length()) ) && (conf_dir[len-1] != '/') )
  {
  conf_dir += "/";
  }
  
//  cout << "Frontend::callback_browse_config\n";
f->action = F_FILE_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->config_entry;
f->get_a_name(widget, f, "Choose CONFIGURATION file", (char *)conf_dir.c_str(), FALSE );
}

void Frontend::callback_browse_mount_point(GtkWidget *widget, Frontend *f)
{
cout << "Frontend::callback_browse_mount_point\n";
f->action = F_DIRECTORY_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->mount_point_entry;
f->get_a_name(widget, f, "Choose MOUNT POINT for Backup Device", "/", TRUE);
}

void Frontend::callback_browse_destination(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_destination\n";

f->action = F_DIRECTORY_PATHNAME;
f->clear_labels(f);

string dirname = f->spiffa->backup_destination + "/";
if (!isdir((char *)dirname.c_str())) dirname = "/";

f->entry_for_select = f->backup_destination_entry;
f->get_a_name(widget, f, "Choose DESTINATION directory for Backup", (char *)dirname.c_str(), TRUE);
}


void Frontend::callback_browse_new_source(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_new_source\n";
f->action = F_FILE_OR_DIRECTORY_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->new_source_entry;
f->get_a_name(widget, f, "Choose a new source directory or file", "/", FALSE);
} 

void Frontend::callback_browse_new_omission(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_new_omission\n";
f->action = F_FILE_OR_DIRECTORY_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->new_omission_entry;
f->get_a_name(widget, f, "Choose a new directory or file for omission", "/", FALSE);
}



void Frontend::callback_browse_sh_pn(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_sh_pn\n";
f->action = F_FILE_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->sh_pn_entry;
f->get_a_name(widget, f, "Choose pathname of sh", "/", FALSE);
}

void Frontend::callback_browse_cat_pn(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_cat_pn\n";
f->action = F_FILE_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->cat_pn_entry;
f->get_a_name(widget, f, "Choose pathname of cat", "/", FALSE);
}

void Frontend::callback_browse_gzip_pn(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_gzip_pn\n";
f->action = F_FILE_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->gzip_pn_entry;
f->get_a_name(widget, f, "Choose pathname of gzip", "/", FALSE);
}

void Frontend::callback_browse_bzip2_pn(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_bzip2_pn\n";

f->action = F_FILE_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->bzip2_pn_entry;
f->get_a_name(widget, f, "Choose pathname of bzip2", "/", FALSE);
}

void Frontend::callback_browse_ln_pn(GtkWidget *widget, Frontend *f)
{
//  cout << "Frontend::callback_browse_ln_pn\n";

f->action = F_FILE_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->ln_pn_entry;
f->get_a_name(widget, f, "Choose pathname of ln", "/", FALSE);
}

void Frontend::callback_browse_mtab_pn(GtkWidget *widget, Frontend *f)
{
cout << "Frontend::callback_browse_mtab_pn\n";
f->action = F_FILE_PATHNAME;
f->clear_labels(f);
f->entry_for_select = f->mtab_pn_entry;
f->get_a_name(widget, f, "Choose pathname of mtab", "/", FALSE);
}

void Frontend::callback_pathnames_defaults(Frontend *f)
{
f->clear_labels(f);

gtk_entry_set_text(GTK_ENTRY(f->sh_pn_entry), "/bin/sh");
gtk_entry_set_text(GTK_ENTRY(f->cat_pn_entry), "/bin/cat");
gtk_entry_set_text(GTK_ENTRY(f->gzip_pn_entry), "/bin/gzip");
gtk_entry_set_text(GTK_ENTRY(f->bzip2_pn_entry), "/usr/bin/bzip2");
gtk_entry_set_text(GTK_ENTRY(f->ln_pn_entry), "/bin/ln");
gtk_entry_set_text(GTK_ENTRY(f->mtab_pn_entry), "/etc/mtab");
gtk_entry_set_text(GTK_ENTRY(f->browser_pn_entry), "htmlview %s");
}

void Frontend::callback_browse_docs(Frontend *f)
{
//cout << "Frontend::callback_browse_docs\n";

string command;

f->clear_labels(f);

if (f->spiffa->browser_pname.length() == 0)
  {
  gtk_label_set(GTK_LABEL(f->gui_info_label), "HTML browser command set");
  return;
  }

char *p = (char *)f->spiffa->browser_pname.c_str();

// cout << "Frontend::callback_browse_docs p= " << p << "\n";

if (p && *p)
  {
  while (*p)
    {
    if (STREQN(p, "%s", 2))
      {
        command += DOCDIR "/SPIFFA.html";
        p++;
      }
    else command += *p;
    p++;
    }
  }
if (command.length())
  {
  if (command[command.length()-1] != '&') command += " &";

  //cout << "Frontend::callback_browse_docs - command= " << command << "\n";

  system(command.c_str()); 
  }
}

void Frontend::callback_add_new_source(GtkWidget *widget, Frontend *f)
{
  GtkTreeModel* model;
  gboolean valid;
  GtkTreeIter iter;
  GtkTreeIter sibling;
  gchar *str;
  string msg;

  string src = (gchar *)gtk_entry_get_text(GTK_ENTRY(f->new_source_entry));
  f->clear_labels(f);

  //  cout << "Frontend::callback_add_new_source\n";

  // if src ends with a slash, remove it

  while (src.length()>1 && src[src.length()-1]=='/') src = src.substr(0, src.length()-1);
  if (src.length()==0)
    {
      msg = "Can't add \"\" to backup list : no source.";
      gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
      return;
    }

  if ( (src == "/proc") || (src == "/sys") )
    {
      msg = "Can't add \"" + src;
      msg += "\" to backup list : not permitted.";
      gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
      return;
    }

  // Must be an absolute pathname, can't begin anything other than a slash
  if (src[0] != '/')
    {
      msg = "Can't add \"" + src;
      msg += "\" to backup list : not an absolute pathname.";
      gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
      return;
    }

  // We need to be able to make it active again
  gtk_entry_set_text(GTK_ENTRY(f->new_source_entry), "");

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->source_treeview));
  valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

  while (valid)
    {
    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
       0, // the column #,
       &str,
       -1);

    if (src == str)   // If already in the list
      {
      msg = "Can't add \"" + src;
      msg += "\" to backup list : already present.";
      gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
      return;  // Don't add  if already present
      }

    if (STREQN(str, (char *)src.c_str(), strlen(str)))
      {
      msg = "Can't add \"" + src;
      msg += "\" to backup list : included in existing entry \"";
      msg += str;
      msg += "\"";
      gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
      return;  // Don't add  if already included by an existing entry
      }

    if (STREQN((char *)src.c_str(), str, src.length()))
      {
      // Remove the existing entry and replace it with this because this includes the old one
      // Note that it only has to be removed I think! - because it is going to be added anyway.


      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);

      msg = "Removed \"";
      msg += str;
      msg += "\" from backup list : included in new entry \"";
      msg += src;
      msg += "\"";
      gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());

      // cout << "Frontend::callback_add_new_source - " << msg << endl;
      }

    else if (strcasecmp(src.c_str(), str) < 0) // ignore case to find where to put it
      {
      // INSERT IT HERE
      gtk_list_store_insert_before(GTK_LIST_STORE(model),
                                &sibling,
                                &iter); 
      gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0, src.c_str(), -1);
      break;
      }
  
    valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
    }

  if (!valid)
    {
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set (GTK_LIST_STORE(model), &iter, 0, src.c_str(), -1);
    }
}

void Frontend::callback_delete_source(GtkWidget *widget, Frontend *f)
{
GtkTreeIter iter;
GtkTreeSelection *select;
GtkTreeModel *model;

//cout << "Frontend::callback_delete_source\n";
f->clear_labels(f);

select = gtk_tree_view_get_selection (GTK_TREE_VIEW(f->source_treeview));
gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);

if ( (gtk_tree_selection_get_selected(select, &model, &iter)) )
  {
  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
  }
}

void Frontend::callback_add_new_omission(GtkWidget *widget, Frontend *f)
{
GtkTreeModel* model;
gboolean valid;
GtkTreeIter iter;
GtkTreeIter sibling;
gchar *str;
string msg;

string src = (gchar *)gtk_entry_get_text(GTK_ENTRY(f->new_omission_entry));
f->clear_labels(f);

// If it ends in "/" the the directory contents are prohibited
// but the directory name may be included.
// If it doesn't end in "/" both the directory name
// and its contents are omitted.

//  cout << "Frontend::callback_new_omision\n";

if (!src.length()) return;

// We need to be able to make it active again
gtk_entry_set_text(GTK_ENTRY(f->new_omission_entry), "");

model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->omit_treeview));

valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

while (valid)
  {
  gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                    0, // the column #,
                    &str,
                    -1);
  
  if (src == str)
    {
    msg = "Can't add \"" + src;
    msg += "\" : already present.";
    gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
    return;  // Don't add  if already present
    }

  if (STREQN(str, (char *)src.c_str(), strlen(str)))
    {
    msg = "Can't add \"" + src;
    msg += "\" : included in existing entry \"";
    msg += str;
    msg += "\"";
    gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
    return;  // Don't add  if already included by an existing entry
    }

  if (STREQN((char *)src.c_str(), str, src.length()))
    {
    // Remove the existing entry and replace it with this because this includes the old one
    // Note that it only has to be removed I think! - because it is going to be added anyway.

    // Not sure if this works...
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);

    msg = "Removed \"";
    msg += str;
    msg += "\" : included in new entry \"";
    msg += src;
    msg += "\"";
    gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());

    //cout << "Frontend::callback_add_new_source - " << msg << endl;
    }

  else if (strcasecmp(src.c_str(), str) < 0)
    {
    // INSERT IT HERE
    gtk_list_store_insert_before(GTK_LIST_STORE(model),
               &sibling,
               &iter); 
    gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0, src.c_str(), -1);
    break;
    }
  
  valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
  }

if (!valid)
  {
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set (GTK_LIST_STORE(model), &iter, 0, src.c_str(), -1);
  }
}

void Frontend::callback_delete_omission(GtkWidget *widget, Frontend *f)
{
GtkTreeIter iter;
GtkTreeSelection *select;
GtkTreeModel *model;

//cout << "Frontend::callback_delete_omission\n";

 f->clear_labels(f);

select = gtk_tree_view_get_selection (GTK_TREE_VIEW(f->omit_treeview));
gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

if ( (gtk_tree_selection_get_selected(select, &model, &iter)) )
  {
    string proc = "/proc";
    string sys =  "/sys";
    gchar *str;

    //    cout << "Frontend::callback_delete_omission - GOT delete selection\n";

    // If it is /proc it should not be removed...
    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
           0, // the column #,
           &str,
           -1);
    if ((str == proc) || (str == sys))
      {
      string msg = "Can't delete \"";
      msg +=  str;
      msg += "\" : backup would be unsafe";
      //  cout << "Frontend::callback_delete_omission - NOT DELETING \"" << (char *)str << "\"" << endl;
      gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
      }
    else gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
  }
}

void Frontend::callback_add_new_user(GtkWidget *widget, Frontend *f)
{
  GtkTreeModel* model;
  gboolean valid;
  GtkTreeIter iter;
  GtkTreeIter sibling;
  gchar *str;
  string src = (gchar *)gtk_entry_get_text(GTK_ENTRY(f->new_user_entry));
  f->clear_labels(f);

  if (!src.length()) return;

  //  cout << "Frontend::callback_new_user\n";

  // We need to be able to make it active again
  gtk_entry_set_text(GTK_ENTRY(f->new_user_entry), "");

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->user_treeview));

  valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

  while (valid)
    {
      gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
       0, // the column #,
       &str,
       -1);
  
      if (src == str)
  {
    string msg = "Can't add \"" + src;
    msg += "\" : already present.";
    gtk_label_set(GTK_LABEL(f->gui_info_label), msg.c_str());
    return;  // Don't add  if already present
  }
    
      if (strcasecmp(src.c_str(), str) < 0)
  {
    // INSERT IT HERE
    gtk_list_store_insert_before(GTK_LIST_STORE(model),
               &sibling,
               &iter); 
    gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0, src.c_str(), -1);
    break;
  }
  
      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
    }

  if (!valid)
    {
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set (GTK_LIST_STORE(model), &iter, 0, src.c_str(), -1);
    }
}

void Frontend::callback_delete_user(GtkWidget *widget, Frontend *f)
{
GtkTreeIter iter;
GtkTreeSelection *select;
GtkTreeModel *model;

//cout << "Frontend::callback_delete_user\n";

 f->clear_labels(f);

select = gtk_tree_view_get_selection (GTK_TREE_VIEW(f->user_treeview));
gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

if ( (gtk_tree_selection_get_selected(select, &model, &iter)) )
  {
    //  cout << "Frontend::callback_delete_user - GOT delete selection\n";
  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
  }
}

void Frontend::save_backup_data(Frontend *f)
{
f->clear_labels(f);

if (f->timer)
  {
  f->report("save backup options", f);
  }
else
  {
  GtkTreeModel* model;
  list<string>::iterator i;

  GtkTreeIter iter;
  gboolean valid;
  gchar *str;

  // Delete existing backup_sources
  f->spiffa->sources.clear();

  // Read backup_sources into Conf::backup_sources
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->source_treeview));

  valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

  while (valid)
    {
    // Walk through the list, reading each row
    // Make sure you terminate calls to gtk_tree_model_get()
    // with a '-1' value
    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
           0, // the column #,
           &str,
           -1);

    f->spiffa->sources.push_back(str);

    valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
    }

  // Read mount point and destination directory
  f->spiffa->mount_point = gtk_entry_get_text(GTK_ENTRY(f->mount_point_entry));
  f->spiffa->backup_destination = gtk_entry_get_text(GTK_ENTRY(f->backup_destination_entry));

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(f->gz_radio)))
    {
    f->spiffa->compression = "GZ";
    }
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(f->bz2_radio)))
    {
    f->spiffa->compression = "BZ2";
    }
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(f->gz_bz2_radio)))
    {
    f->spiffa->compression = "GZ_BZ2";
    }
  else //default
    {
    f->spiffa->compression = "NONE";
    }

  f->spiffa->keep_number = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(f->keep_spinner));


  // Save the configuration via spiffaslave
  f->m->send_message(SP_START_SOURCES, NULL);

  for (i=f->spiffa->sources.begin(); i!=f->spiffa->sources.end(); ++i)
    {
    f->m->send_message(SP_SOURCE, (char *)i->c_str() );
    }
  f->m->send_message(SP_MOUNT_POINT, (char *)f->spiffa->mount_point.c_str() );
  f->m->send_message(SP_DESTINATION, (char *)f->spiffa->backup_destination.c_str() );
  f->m->send_message(SP_COMPRESSION, (char *)f->spiffa->compression.c_str() );
  f->m->send_message(SP_KEEP_NUMBER, (char *)(TFile::itoa(f->spiffa->keep_number)).c_str() );
  }
}


void Frontend::save_other_data(Frontend *f)
{
  f->clear_labels(f);
  if (f->timer)
    {
      f->report("save other options", f);
    }
  else
    {
      GtkTreeModel* model;

      list<string>::iterator i;

      GtkTreeIter iter;
      gboolean valid;
      gchar *str;

      f->spiffa->omissions.clear();

      // Read Sources to omit into Conf::backup_deny
      model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->omit_treeview));

      valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

      while (valid)
        {
        // Walk through the list, reading each row
        // Make sure you terminate calls to gtk_tree_model_get()
        // with a '-1' value
        gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                            0, // the column #,
                            &str,
                            -1);
        f->spiffa->omissions.push_back(str);

        valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
        }

      f->m->send_message(SP_START_OMISSIONS, NULL);

      for (i=f->spiffa->omissions.begin(); i!=f->spiffa->omissions.end(); ++i)
        {
        f->m->send_message(SP_OMIT, (char *)i->c_str());
        }

      f->spiffa->save_existing_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(f->save_files_check_button));
      f->m->send_message(SP_SAVE_EXISTING_FILES, (char *)((f->spiffa->save_existing_files) ? "1" : "0") );
      f->spiffa->restore_orphans = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(f->restore_orphan_files_check_button));
      f->m->send_message(SP_RESTORE_ORPHANS, (char *)((f->spiffa->restore_orphans) ? "1" : "0") );

      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(f->new_log)))
        {
        f->spiffa->log_type = "NEW";
        }
      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(f->append_log)))
        {
        f->spiffa->log_type = "APPEND";
        }
      else //default
        {
        f->spiffa->log_type = "NONE";
        }
      f->m->send_message(SP_LOG_TYPE, (char *)f->spiffa->log_type.c_str());
    }
}

void Frontend::save_user_data(Frontend *f)
{
  f->clear_labels(f);
  if (f->timer)
    {
      f->report("save users", f);
    }
  else
    {
      GtkTreeIter iter;
      gboolean valid;
      gchar *str;
      GtkTreeModel* model;
      list<string>::iterator i;

      // Users only if available, i.e. root
      if (f->user_treeview)
        {
        f->spiffa->users.clear();

        model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->user_treeview));
        valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

        while (valid)
          {
          gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                              0, // the column #,
                              &str,
                              -1);  
          f->spiffa->users.push_back(str);

          valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
          }


        f->m->send_message(SP_START_USERS, NULL);

        for (i=f->spiffa->users.begin(); i!=f->spiffa->users.end(); ++i)
          {
          f->m->send_message(SP_USER, (char *)i->c_str());
          }
        }
    }
}

void Frontend::save_pathnames(Frontend *f)
{
// These are only available for root
  f->clear_labels(f);
  if (f->timer)
    {
      f->report("save pathnames", f);
    }
  else
    {
    if (f->user_treeview)
      {
      f->spiffa->sh_pname = gtk_entry_get_text(GTK_ENTRY(f->sh_pn_entry));
      f->spiffa->cat_pname = gtk_entry_get_text(GTK_ENTRY(f->cat_pn_entry));
      f->spiffa->gzip_pname = gtk_entry_get_text(GTK_ENTRY(f->gzip_pn_entry));
      f->spiffa->bzip2_pname = gtk_entry_get_text(GTK_ENTRY(f->bzip2_pn_entry));
      f->spiffa->ln_pname = gtk_entry_get_text(GTK_ENTRY(f->ln_pn_entry));
      f->spiffa->mtab_pname = gtk_entry_get_text(GTK_ENTRY(f->mtab_pn_entry));
      f->spiffa->browser_pname = gtk_entry_get_text(GTK_ENTRY(f->browser_pn_entry));

      f->m->send_message(SP_SH_PN, (char *)f->spiffa->sh_pname.c_str());
      f->m->send_message(SP_CAT_PN, (char *)f->spiffa->cat_pname.c_str());
      f->m->send_message(SP_GZIP_PN, (char *)f->spiffa->gzip_pname.c_str());
      f->m->send_message(SP_BZIP2_PN, (char *)f->spiffa->bzip2_pname.c_str());
      f->m->send_message(SP_LN_PN, (char *)f->spiffa->ln_pname.c_str());
      f->m->send_message(SP_MTAB_PN, (char *)f->spiffa->mtab_pname.c_str());
      f->m->send_message(SP_BROWSER_PN, (char *)f->spiffa->browser_pname.c_str());
      }
    }
}

void Frontend::callback_save_backup_data(GtkWidget *widget, Frontend *f)
{
f->clear_labels(f);
if (f->timer)
  {
  f->report("save backup options", f);
  }
else
  {
  f->save_backup_data(f);
  f->m->send_message(SP_CONF_DONE, NULL);
  }
}

void Frontend::callback_save_other_data(GtkWidget *widget, Frontend *f)
{
f->clear_labels(f);
if (f->timer)
  {
  f->report("save other options", f);
  }
else
  {
  f->save_other_data(f);
  f->m->send_message(SP_CONF_DONE, NULL);
  }
}

void Frontend::callback_save_user_data(GtkWidget *widget, Frontend *f)
{
f->clear_labels(f);
if (f->timer)
  {
  f->report("save users", f);
  }
else
  {
  f->save_user_data(f);
  f->m->send_message(SP_CONF_DONE, NULL);
  }
}

void Frontend::callback_save_pathnames(GtkWidget *widget, Frontend *f)
{
f->clear_labels(f);
if (f->timer)
  {
  f->report("save pathnames", f);
  }
else
  {
  f->save_pathnames(f);
  f->m->send_message(SP_CONF_DONE, NULL);
  }
}

void Frontend::callback_save(GtkWidget *widget, Frontend *f)
{
  f->clear_labels(f);

  if (f->timer)
    {
      f->report("\"save\" all the data", f);
    }
  else
    {
    string action_msg = "Saving default configuration file";

    // Where the image is displayed
    f->rainbow_world_image = f->rainbow_world_image_bu;

    f->spiffa->slave_working = TRUE;
    f->action = F_SAVE;

    f->save_user_data(f);
    f->save_backup_data(f);
    f->save_other_data(f);
    f->save_pathnames(f);

    gtk_label_set(GTK_LABEL(f->action_label), action_msg.c_str());

    f->m->send_message(SP_CONF_DONE, NULL);
    f->start_timer(f);
    }
}

void Frontend::callback_save_as(GtkWidget *widget, Frontend *f)
{
unsigned int len;
string conf_dir = SYSCONFDIR;

if ( ( (len=conf_dir.length()) ) && (conf_dir[len-1] != '/') )
  {
  conf_dir += "/";
  }

f->clear_labels(f);
  
if (f->timer)
  {
  f->report("\"save as\" all the data", f);
  }
else
  {
  f->action = F_SAVE_AS;
  f->entry_for_select = NULL;
  f->get_a_name(widget, f, "Save Configuration As", (char *)conf_dir.c_str(), FALSE);
  }
}

void Frontend::callback_load(GtkWidget *widget, Frontend *f)
{
f->clear_labels(f);

if (f->timer)
  {
  f->report("load selected configuration data", f);
  }
else
  {
  unsigned int len;
  string conf_dir = SYSCONFDIR;

  if ( ( (len=conf_dir.length()) ) && (conf_dir[len-1] != '/') )
    {
    conf_dir += "/";
    }

  f->action = F_LOAD;
  f->entry_for_select = NULL;
  f->get_a_name(widget, f, "Load CONFIGURATION file for edit", (char *)conf_dir.c_str(), FALSE);
  }
}

void Frontend::clear_labels(Frontend *f)
{
gtk_label_set(GTK_LABEL(f->action_label), "");
gtk_label_set(GTK_LABEL(f->point_label), "");
gtk_label_set(GTK_LABEL(f->path_label), "");
gtk_label_set(GTK_LABEL(f->file_label), "");
gtk_label_set(GTK_LABEL(f->status_label), "");
gtk_label_set(GTK_LABEL(f->status_extra_label), "");
gtk_label_set(GTK_LABEL(f->gui_info_label), "");
gtk_label_set(GTK_LABEL(f->time_label), "");
gtk_label_set(GTK_LABEL(f->elapsed_time_label), "");
}

gboolean Frontend::slave_busy(gpointer gdata)
{
string elapsed_str;

Frontend *f = (Frontend *)gdata;

time2string( (time(NULL) -f->start_time), elapsed_str);

if (f->action < F_LOAD)
  {
  string elapsed_str;
  time2string( (time(NULL) -f->start_time), elapsed_str);
  gtk_label_set(GTK_LABEL(f->time_label), elapsed_str.c_str());
  }
f->bw->busy_world();
f->m->get_messages();

if  (!f->spiffa->slave_working)
    {
    // cout << "Frontend::slave_busy - slave has FINISHED\n";
    delete f->bw;
    f->bw = NULL;
    f->stop_timer();
    }
return TRUE;
}

void  Frontend::stop_timer(void)
{
  if (timer)
    {
    gtk_widget_hide(rainbow_world_image);
    g_source_remove(timer);
    timer = 0;
   }
}


void Frontend::start_timer(Frontend *f)
{
if (!(f->timer))
    {
    if (f->action < F_LOAD)
        {
        start_time = time(NULL);
        gtk_label_set(GTK_LABEL(f->elapsed_time_label), "ELAPSED TIME: ");
        gtk_widget_show(rainbow_world_image);
        }
  
    f->bw = new BusyWorld(f);  
    timer = g_timeout_add(200, f->slave_busy, f);
    }
    
}
