/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _FRONTEND_H
#define _FRONTEND_H

#include <iostream>
#include <list>
#include <gtk/gtk.h>

#include "complain.h"
#include "../common/messages.h"
#include "spiffa.h"
#include "busy.h"

using namespace std;

enum {
  F_BACKUP,
  F_UPDATE,
  F_RESTORE,
  F_LOAD,
  F_SAVE,
  F_SAVE_AS,
  F_RESTORE_POINT,
  F_FILE_PATHNAME,
  F_DIRECTORY_PATHNAME,
  F_FILE_OR_DIRECTORY_PATHNAME,
  };

enum
{
  COL_NAME = 0,
  COL_CHECKBUTTON,
  NUM_COLS
};

class Frontend
  {
    friend class SpiffaMessages;  // Allow SpiffaMesssages to access everything

  public:
    Frontend(int *argc, char **argv[], class SpiffaMessages *msg, class Spiffa *s);
    ~Frontend(void);

    static int action;

    GtkWidget *rainbow_world_image;
    GdkPixbuf *spiffa_icon_pixbuf;
    GtkWidget *window;
    static GtkWidget *gui_info_label;

  private:
    GtkWidget *add_page(GtkWidget *notebook, char *title);
    void fill_backup_actions_page(GtkWidget *vbox);
    void fill_backup_options_page(GtkWidget *vbox);
    void fill_other_options_page(GtkWidget *vbox);
    void fill_users_page(GtkWidget *vbox);
    void fill_pathnames_page(GtkWidget *vbox);
    void fill_about_page(GtkWidget *vbox);
    void clear_labels(Frontend *f);
    void report(char *desired_action, Frontend *f);
    bool get_default_restore_point(string *backup_destination, string *restore_point);
    bool get_restore_point_pathname(string *from, string *pathname);
    GtkWidget *get_list_store(list<string> mlist, gchar *title);

    GtkTreeModel *create_model(list<string> mlist, gchar *title);

    void add_columns (GtkTreeView *treeview, gchar *title);

    void get_a_name(GtkWidget *widget, Frontend *f, char *title, char *selection, bool dir_only);
    static void insert_selected(GtkWidget *widget,  Frontend *f);
    static void close_file_requester(GtkWidget *widget,  Frontend *f);

    static void do_quit(GtkWidget *widget, Complain *c);
    static void no_quit(GtkWidget *widget, Complain *c);

    static gint callback_quit(GtkWidget *widget, GdkEventButton *event, Frontend *f);
    static void callback_backup(GtkWidget *widget, Frontend *f);
    static void callback_update(GtkWidget *widget, Frontend *f);
    static void callback_restore(GtkWidget *widget, Frontend *f);
    
    static void callback_restore_from(GtkWidget *widget, Frontend *f);
    static void callback_browse_from(GtkWidget *widget, GdkEventButton *event, Frontend *f);
    static void callback_browse_config(GtkWidget *widget, Frontend *f);
    static void callback_add_new_source(GtkWidget *widget, Frontend *f);
    static void callback_browse_new_source(GtkWidget *widget, Frontend *f);
    static void callback_browse_mount_point(GtkWidget *widget, Frontend *f);
    static void callback_browse_destination(GtkWidget *widget, Frontend *f);
    static void callback_delete_source(GtkWidget *widget, Frontend *f);
    static void callback_add_new_omission(GtkWidget *widget, Frontend *f);
    static void callback_delete_omission(GtkWidget *widget, Frontend *f);
    static void callback_browse_new_omission(GtkWidget *widget, Frontend *f);
    static void callback_add_new_user(GtkWidget *widget, Frontend *f);
    static void callback_delete_user(GtkWidget *widget, Frontend *f);
    static void callback_browse_sh_pn(GtkWidget *widget, Frontend *f);
    static void callback_browse_cat_pn(GtkWidget *widget, Frontend *f);
    static void callback_browse_gzip_pn(GtkWidget *widget, Frontend *f);
    static void callback_browse_bzip2_pn(GtkWidget *widget, Frontend *f);
    static void callback_browse_ln_pn(GtkWidget *widget, Frontend *f);
    static void callback_browse_mtab_pn(GtkWidget *widget, Frontend *f);
    static void callback_pathnames_defaults(Frontend *f);
    static void callback_browse_docs(Frontend *f);

    static void selection_exposed(GtkWidget *widget, GdkEvent *event, Frontend *f);
    static void quit_selection(GtkWidget *widget, Frontend *f);
    static void selection_ok(GtkWidget *widget, Frontend *f);   
    static void selection_cancel(GtkWidget *widget, Frontend *f);
    
    void traverse_descendents(GtkTreeIter *parent, string *parent_name, Frontend *f);
    void fix_descendents(GtkTreeIter *parent, gboolean mark_flag);
    static void toggle_func(GtkCellRendererToggle *cell_renderer, gchar *path, Frontend *f);
    void populate_directory(GtkTreeStore *treestore, GtkTreeIter *parent, string *dirpathname);
    void populate_tree(GtkTreeStore *treestore, string *restore_point);
    GtkTreeModel *create_and_fill_model(Frontend *f, string *restore_point);
    GtkWidget *create_view_and_model (Frontend *f, string *restore_point);
    void make_selection(Frontend *f);

    void save_backup_data(Frontend *f);
    void save_other_data(Frontend *f);
    void save_user_data(Frontend *f);
    void save_pathnames(Frontend *f);

    static void callback_save_backup_data(GtkWidget *widget, Frontend *f);
    static void callback_save_other_data(GtkWidget *widget, Frontend *f);
    static void callback_save_user_data(GtkWidget *widget, Frontend *f);
    static void callback_save_pathnames(GtkWidget *widget, Frontend *f);
    static void callback_save_as(GtkWidget *widget, Frontend *f);
    static void callback_save(GtkWidget *widget, Frontend *f);
    static void callback_load(GtkWidget *widget, Frontend *f);

    static gboolean slave_busy(gpointer gdata);
    void stop_timer(void);
    void start_timer(Frontend *f);

    GtkTooltips *tooltips;
    GtkWidget *notebook;
    GtkWidget *source_treeview;
    GtkWidget *omit_treeview;
    GtkWidget *user_treeview;

    GtkWidget *new_source_entry;
    GtkWidget *new_omission_entry;
    GtkWidget *new_user_entry;
    GtkWidget *from_entry;
    GtkWidget *config_entry;
    GtkWidget *mount_point_entry;
    GtkWidget *backup_destination_entry; 

    // pathnames page
    static GtkWidget *sh_pn_entry; 
    static GtkWidget *cat_pn_entry; 
    static GtkWidget *gzip_pn_entry; 
    static GtkWidget *bzip2_pn_entry; 
    static GtkWidget *ln_pn_entry;
    static GtkWidget *mtab_pn_entry;
    static GtkWidget *browser_pn_entry; 

    GSList *compression_group;
    GtkWidget *none_radio;
    GtkWidget *gz_radio;
    GtkWidget *bz2_radio;
    GtkWidget *gz_bz2_radio;

    GSList *log_group;
    GtkWidget *new_log;
    GtkWidget *append_log;
    GtkWidget *no_log;


    GtkWidget *keep_spinner; 
    GtkWidget *save_files_check_button;
    GtkWidget *restore_orphan_files_check_button;

    SpiffaMessages *m;
    class Spiffa *spiffa;

    GtkWidget *filew;
    GtkWidget *entry_for_select;
    GtkWidget *select_dialog;
    GtkWidget *restore_selection_label;
    
    static guint timer;

    class BusyWorld *bw;
    
    GtkStyle *black_text_style;
    GtkStyle *red_text_style;
    GtkStyle *blue_text_style;
    GtkStyle *green_text_style;
    GtkStyle *purple_text_style;
    GtkStyle *orange_text_style;

    GtkWidget *rainbow_world_image_bu;
    GtkWidget *rainbow_world_image_rs;

    static GdkColor red;
    static GdkColor blue;
    static GdkColor green;
    static GdkColor yellow;
    static GdkColor purple;
    static GdkColor orange;
    static GdkColor cyan;
    static GdkColor black;
    static GdkColor white;

    static const guint8 spiffa_pixbuf[];

    static GtkWidget *action_label;
    static GtkWidget *point_label;
    static GtkWidget *path_label;
    static GtkWidget *file_label;
    static GtkWidget *status_label;
    static GtkWidget *status_extra_label;

    static GtkWidget *elapsed_time_label;
    static GtkWidget *time_label;
    
    GtkWidget *restore_point_label; // Restore Point or Restore Selection
    
    GtkTreeModel *model;
    GtkTreeStore *treestore;
    list<string> restore_selection;

    guint selection_exposed_signal;
    
    time_t start_time;

  protected:

  };

#endif // _FRONTEND_H
