/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>

#include <pwd.h>
#include <unistd.h>
#include <getopt.h>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <errno.h>

#include "spiffa.h"
#include "frontend.h"
#include "../common/misc.h"
#include "../common/messages.h"
#include "../common/tfile.h"
#include "../common/version.h"
#include "complain.h"


#define UNLESS(x) if(!(x))
#define FOREVER for(;;)
#define UNTIL(x) while(!(x))

using namespace std;
using namespace misc;

string Spiffa::browser_pname;

Spiffa::~Spiffa(void)
{
//  cout << "~Spiffa:\n";

// Clear the lists
sources.clear();
omissions.clear();
users.clear();

// Remove the message queue
if (qid >= 0)
  {
  msgctl(qid, IPC_RMID, NULL);
  }
}

void Spiffa::start_slave(pid_t pid)
{
  /**************************************************
  cout << "Spiffa::start_slave:" << endl;
  cout << "version_txt  = " << version_txt << endl;
  cout << "EPREFIX      = " << EPREFIX << endl;
  cout << "PREFIX       = " << PREFIX << endl;
  cout << "BINDIR       = " << BINDIR << endl;
  cout << "SBINDIR      = " << SBINDIR << endl;
  cout << "DATADIR      = " << DATADIR << endl;
  cout << "SYSCONFDIR   = " << SYSCONFDIR << endl;
  cout << "LOCALSTATEDIR= " << LOCALSTATEDIR << endl;
  cout << "INFODIR      = " << INFODIR << endl;
  cout << "MANDIR       = " << MANDIR << endl;

  cout << "LOGDIR       = " << LOGDIR << endl;
  cout << "RUNDIR       = " << RUNDIR << endl;
  cout << "DOCDIR       = " << DOCDIR << endl;
  **************************************************/

  string command = BINDIR;
  command += "/spiffaslave ";
  command += TFile::itoa(pid);
  command += " ";
  command +=  TFile::itoa(qid);
  command += " &";

  system(command.c_str());
}

void Spiffa::complain_abort(GtkWidget *widget, Complain *c)
{
gtk_main_quit();
}

Spiffa::Spiffa(int argc, char *argv[])
{
key_t     key;
pid_t     pid;                   // process id of spiffa (this process)

//  cout << "Spiffa:\n";

slave_pid = 0;
quit_flag = no_permission_flag = FALSE;
keep_number = 1;
save_existing_files = FALSE;

/********************************
for (int n=0; n<argc; ++n)
  {
  cout << *argv[n] << endl;
  }
********************************/

pid = key = getpid();

FOREVER
  {
  qid = msgget(key, IPC_CREAT | IPC_EXCL | 0666);

  if (qid >= 0) break;

  key++;
  if (key > 32000) key = 2;
  if (key == pid)
    {
    string msg = "Can't allocate a message key : ";
    msg += strerror(errno);
    msg += "\nYou probably have to reboot\n";

    cerr << msg << endl;

    gtk_init (&argc, &argv);

    Complain *c = new Complain;
    c->complain(c->CN_ABORT,
        "Spiffa",
        (char *)msg.c_str(),
        complain_abort,
        complain_abort,
        this);
    gtk_main();

    return;
    }
  } // End of FOREVER

//cout << "Spiffa: - GOT pid= " << pid << " : qid= " << qid << endl;

SpiffaMessages m(qid, 0, pid, this);

start_slave(pid);

//cout << "Spiffa: - Returned from start_slave" << endl;

conf_done = FALSE;
for (int n=0; n<1000; ++n)  // The initial messages should all come here
  {
  m.get_messages();

  if (no_permission_flag) break;

  if (slave_pid) break;

  usleep(500);
  }

if (no_permission_flag && argc==1)
  {
  gtk_init (&argc, &argv);

  Complain *c = new Complain;
    
  c->complain(c->CN_ABORT,
     "Spiffa",
     (char *)no_permission_txt.c_str(),
     complain_abort,
     complain_abort,
     this);
     gtk_main();
  }
else if (no_permission_flag)
  {
  cerr << no_permission_txt << endl;
  return;
  }
else
  {
  if (slave_pid == 0)
    {
    cerr << "Spiffa: NO REPLY FROM spiffaslave\n";
    return;
    }
  //cout << "Spiffa: conf_done= " << conf_done << endl;

  if (!(conf_done))
    {
    for (int n=0; n<1000; ++n)
      {
      m.get_messages();
      if (conf_done) break;
      usleep(500);
      }
    }

  if (!(conf_done))
    {
    cerr << "Spiffa: slave didn't give us conf!\n";
    }
  m.send_mtype(slave_pid);

  //cout << "Spiffa: spiffaslave IS ALIVE,  argc= " << argc << "\n";
  //cout << "Spiffa: spiffaslave pid= " << slave_pid << "\n";

  if (argc == 1)
    {
    // Using GUI
    Frontend::Frontend(&argc, &argv, &m, this);
    }
  else
    {
    // NOT using GUI so parse the args
    bool b, i, r;
    string file;
    string point;

    int c;
    // int digit_optind = 0;

    b=i=r=FALSE;

    FOREVER
      {
      // int this_option_optind = optind ? optind : 1;
      int option_index = 0;
      static struct option long_options[] =
        {
          {"backup", 0, 0, 'b'},
          {"restore", 0, 0, 'r'},
          {"file", 1, 0, 'f'},
          {"point", 1, 0, 'p'},
          {"incremental", 0, 0, 'i'},
          {"update", 0, 0, 'u'},
          {0, 0, 0, 0}
        };

      c = getopt_long(argc, argv, "biruf:p:",
                      long_options, &option_index);
      if (c == -1) break;
      switch (c)
        {
        case 0:
        //printf ("option %s", long_options[option_index].name);
        //if (optarg)
        //  printf (" with arg %s", optarg);
        //printf ("\n");
        break;

        case 'b':
        // cout << "main: got b\n";
        b = TRUE;
        break;

        case 'r':
        // cout << "main: got r\n";
        r = TRUE;
        break;

        case 'f':
        // cout << "main: got f with value " << optarg << "\n";
        file = optarg;
        break;

        case 'i':
        case 'u':
        // cout << "main: got i\n";
        i = TRUE;
        break;

        case 'p':
        point = optarg;
        break;

        case '?':
        break;
      
        default:
        //cout << "main: got UNKNOWN char code " << c << "\n";
        //g_print("\nmain: got UNKNOWN char code %c\n", c);
        break;
        }
      } // END of FOREVER


    //while (optind < argc)
    //  {
    //    printf ("non-option ARGV-elements: ");
    //    while (optind < argc)
    //      printf ("%s ", argv[optind++]);
    //    printf ("\n");
    //  }

    //cout << "main: DONE ARGS\n";

    if (r && (i || b))
      {
      // ARG ERROR, can't backup and restore together!

      cerr << "main: ARG ERROR, can't backup and restore together!\n";
      return;
      }

    if (i)
      {
      // Incremental backup
      if (file.length())
        {
        // Incremental backup using configuration "file"
        //cout << "main: Incremental backup using specified configuration file \"" << file << "\"\n";
        slave_working = TRUE;
        m.send_message(SP_UPDATE, (char *)file.c_str());
        }
      else
        {
        //cout << "main: Incremental backup\n";
        slave_working = TRUE;
        m.send_message(SP_UPDATE, NULL);
        }
      while (slave_working)
        {
        m.get_messages();
        usleep(100);  // Probably should be less than slave's interval
        }
      }
    else if (b)
      {
      if (file.length())
        {
        // Backup using configuration "file"
        //cout << "main: Backup using specified configuration file \"" << file << "\"\n";
        slave_working = TRUE;
        m.send_message(SP_BACKUP, (char *)file.c_str());
        }
      else
        {
        // Normal backup
        //cout << "main: Normal backup\n";
        slave_working = TRUE;
        m.send_message(SP_BACKUP, NULL);
        }

      // Should wait here until slave is done...
      while (slave_working)
        {
        m.get_messages();
        usleep(100);  // Probably should be less than slave's interval
        }
      }
    else if (r)
      {
      string message;

      if (point.length())
        {
        message += point;
        }
      message += "\n";

      if (file.length())
        {
        message += file;
        }

      //cout << "main: Restore(\"" << point << "\", \"" << file << "\"\n";

      slave_working = TRUE;
      m.send_message(SP_RESTORE_FROM, (char *)message.c_str() );

      while (slave_working)
        {
        m.get_messages();
        usleep(100);  // Probably should be less than slave's interval
        }
      }
    }
  }
}
