/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <stdlib.h>
#include <pwd.h>

#include "../common/messages.h"
#include "spiffa.h"
#include "../common/misc.h"
#include "frontend.h"

using namespace std;
using namespace misc;

SpiffaMessages::SpiffaMessages(int q,
             int mst,
             int mrt,
             Spiffa *s):Messages(q, mst)
{
//  cout << "SpiffaMessages:\n";
mrtype = mrt;
spiffa = s;
f = NULL;
}

SpiffaMessages::~SpiffaMessages(void)
{
//  cout << "~SpiffaMessages:\n";
}

void SpiffaMessages::set_frontend(Frontend *frontend)
{
f = frontend;
}

void SpiffaMessages::send_mtype(int type)
{
mstype = type;
}

void SpiffaMessages::get_messages(void)
{
int len;
struct spiffa_msg msg;

while ( (len = msgrcv(qid, &msg, MBUFSIZE, mrtype, IPC_NOWAIT)) >= 0 )
  {
  switch (msg.dtype)
    {
    case SL_ALIVE:
    spiffa->slave_pid = *(pid_t *)msg.data;
    break;

    case SL_POINT:
    if (f)
      {
        gtk_label_set(GTK_LABEL(f->point_label), (char *)msg.data);
      }
    break;

    case SL_PATH:
    if (f)
      {
        gtk_label_set(GTK_LABEL(f->path_label), (char *)msg.data);
        gtk_label_set(GTK_LABEL(f->file_label), "");
      }
    break;

    case SL_FILE:
    if (f)
      {
        gtk_label_set(GTK_LABEL(f->file_label), (char *)msg.data);
      }
    break;

    case SL_CONF_START:
    // Clear everything or default
    spiffa->sources.clear();
    spiffa->omissions.clear();
    spiffa->users.clear();
    spiffa->mount_point = "";
    spiffa->backup_destination = "";
    spiffa->compression = "";
    spiffa->keep_number = 1;
    spiffa->save_existing_files = FALSE;
    spiffa->restore_orphans = FALSE;

    spiffa->sh_pname = "/bin/sh";
    spiffa->cat_pname = "/bin/cat";
    spiffa->gzip_pname = "/bin/gzip";
    spiffa->bzip2_pname = "/usr/bin/bzip2";
    spiffa->ln_pname = "/bin/ln";
    spiffa->mtab_pname = "/etc/mtab";
    spiffa->browser_pname = "htmlview %s";

    spiffa->log_type = "APPEND";

    break;

    case SL_SOURCE:
    spiffa->sources.push_back((char *)msg.data);
    break;

    case SL_OMIT:
    spiffa->omissions.push_back((char *)msg.data);
    break;

    case SL_USER:
    spiffa->users.push_back((char *)msg.data);
    break;

    case SL_MOUNT_POINT:
    spiffa->mount_point = (char *)msg.data;
    break;

    case SL_DESTINATION:
    spiffa->backup_destination = (char *)msg.data;
    break;

    case SL_COMPRESSION:
    spiffa->compression = (char *)msg.data;
    break;

    case SL_KEEP_NUMBER:
    spiffa->keep_number = atoi((char *)msg.data);
    break;

    case SL_SAVE_EXISTING_FILES:
    spiffa->save_existing_files = (STREQ((char *)msg.data, "1" ) ? TRUE : FALSE);
    break;

    case SL_RESTORE_ORPHANS:
    spiffa->restore_orphans = (STREQ((char *)msg.data, "1" ) ? TRUE : FALSE);
    break;

    case SL_SH_PN:
    spiffa->sh_pname = (char *)msg.data;
    break;

    case SL_CAT_PN:
    spiffa->cat_pname = (char *)msg.data;
    break;

    case SL_GZIP_PN:
    spiffa->gzip_pname = (char *)msg.data;
    break;

    case SL_BZIP2_PN:
    spiffa->bzip2_pname = (char *)msg.data;
    break;

    case SL_LN_PN:
    spiffa->ln_pname = (char *)msg.data;
    break;

    case SL_MTAB_PN:
    spiffa->mtab_pname = (char *)msg.data;
    break;

    case SL_BROWSER_PN:
    spiffa->browser_pname = (char *)msg.data;
    break;

    case SL_LOG_TYPE:
    spiffa->log_type = (char *)msg.data;
    break;

    case SL_CONF_DONE:
    if (f)
      {
      if (msg.data && *(char *)msg.data)
        {
        gtk_label_set(GTK_LABEL(f->status_label), (char *)msg.data);
        gtk_label_set(GTK_LABEL(f->gui_info_label), "Loaded configuration unchanged");
        }
      else
        {
        GtkTreeModel* model;
        gboolean valid;
        GtkTreeIter iter;
        list<string>::iterator i;

        // SOURCES
        model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->source_treeview));

        valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
        while (valid)
          {
          gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
          valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
          }
        for (i=spiffa->sources.begin(); i!=spiffa->sources.end(); ++i)
          {
          gtk_list_store_append(GTK_LIST_STORE(model), &iter);
          gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, (gchar *)i->c_str(), -1);
          }

        // OMISSIONS

        model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->omit_treeview));

        valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
        while (valid)
          {
          gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
          valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
          }
        for (i=spiffa->omissions.begin(); i!=spiffa->omissions.end(); ++i)
          {
          gtk_list_store_append(GTK_LIST_STORE(model), &iter);
          gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, (gchar *)i->c_str(), -1);
          }

        // THE REST
        gtk_entry_set_text(GTK_ENTRY(f->mount_point_entry),
                    spiffa->mount_point.c_str());

        gtk_entry_set_text(GTK_ENTRY(f->backup_destination_entry),
                    spiffa->backup_destination.c_str());

        if (spiffa->compression == "GZ")
          {

          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->gz_radio), TRUE);
          }
        else if (spiffa->compression == "BZ2")
          {
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->bz2_radio), TRUE);
          }
        else if (spiffa->compression == "GZ_BZ2")
          {
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->gz_bz2_radio), TRUE);
          }
        else
          {
          // default is no compression
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->none_radio), TRUE);
          }

        gtk_spin_button_set_value(GTK_SPIN_BUTTON(f->keep_spinner), (gdouble)spiffa->keep_number);

        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->save_files_check_button), spiffa->save_existing_files);

        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->restore_orphan_files_check_button), spiffa->restore_orphans);

        if ( STREQ( (getpwuid( getuid() ))->pw_name , "root" ) )
          {
          // USERS
          model = gtk_tree_view_get_model(GTK_TREE_VIEW(f->user_treeview));

          valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
          while (valid)
            {
            gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
            valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
            }

          for (i=spiffa->users.begin(); i!=spiffa->users.end(); ++i)
            {
            gtk_list_store_append(GTK_LIST_STORE(model), &iter);
            gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, (gchar *)i->c_str(), -1);
            }

          // PATHNAMES
          gtk_entry_set_text(GTK_ENTRY(f->sh_pn_entry),
                    spiffa->sh_pname.c_str());
          gtk_entry_set_text(GTK_ENTRY(f->cat_pn_entry),
                    spiffa->cat_pname.c_str());
          gtk_entry_set_text(GTK_ENTRY(f->gzip_pn_entry),
                    spiffa->gzip_pname.c_str());
          gtk_entry_set_text(GTK_ENTRY(f->bzip2_pn_entry),
                    spiffa->bzip2_pname.c_str());
          gtk_entry_set_text(GTK_ENTRY(f->ln_pn_entry),
                    spiffa->ln_pname.c_str());
          gtk_entry_set_text(GTK_ENTRY(f->mtab_pn_entry),
                    spiffa->mtab_pname.c_str());
          gtk_entry_set_text(GTK_ENTRY(f->browser_pn_entry),
                    spiffa->browser_pname.c_str());
          }

        if (spiffa->log_type == "NEW")
          {
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->new_log), TRUE);
          }
        else if (spiffa->log_type == "APPEND")
          {
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->append_log), TRUE);
          }
        else
          {
          // default is no log
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(f->no_log), TRUE);
          }
        gtk_label_set(GTK_LABEL(f->gui_info_label), "New configuration loaded for edit");
        }
      }

    spiffa->conf_done = TRUE;
    spiffa->slave_working = FALSE;
    break;

    case SL_SAVE_DONE:
    if (f)
      {
      if (msg.data && *(char *)msg.data)
        {
        gtk_label_set(GTK_LABEL(f->status_label), (char *)msg.data);
        }
      if (STREQN("ERROR:", (char *)msg.data, 6))
        {
        gtk_label_set(GTK_LABEL(f->gui_info_label), "Configuration not saved");
        }
      else gtk_label_set(GTK_LABEL(f->gui_info_label), "Configuration was saved");
      }

    spiffa->slave_working = FALSE;  
    break;

    case SL_NO_PERMISSION:
    spiffa->no_permission_flag=TRUE;
    spiffa->no_permission_txt = msg.data;
    cerr << (char *)msg.data << endl;
    break;


    case SL_DONE_BACKUP:
    case SL_DONE_RESTORE:
    case SL_DONE_INCREMENTAL_BACKUP:

    spiffa->slave_working = FALSE;
    if (f)
      {
        string status = "";
        string elapsed_str = "";

        char *p = (char *)msg.data;

        while (*p && ((*p)!='\n'))
          {
          if (STRIEQN(p, " : Elapsed Time ", 16))
            {
            p += 16;
            break;
            }
          status += *p++;
          }
        while (*p && ((*p)!='\n'))
          {
          elapsed_str += *p++;
          }

        if (*p) p++;

        gtk_label_set(GTK_LABEL(f->status_label), status.c_str());
        gtk_label_set(GTK_LABEL(f->status_extra_label), p);
        gtk_label_set(GTK_LABEL(f->time_label), elapsed_str.c_str());
      }
    break;

    case SL_FATAL_ERROR:
    {
    if (f)
      {
      gtk_label_set(GTK_LABEL(f->file_label), (char *)msg.data);
      }
    }
    break;

    case SL_TEXT:
    if (f)
      {
        string status = (char *)msg.data;
        gtk_label_set(GTK_LABEL(f->status_label), status.c_str());
      }
    break;

    case SL_QUIT:
    spiffa->quit_flag = TRUE;
    cout << (char *)msg.data << endl;
    break;

    default:
    cerr << "SpiffaMessages::get_messages - GOT UNKNOWN DATA TYPE " << msg.dtype << "\n";
    break;
    }
  }
}
