/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _BACKUP_H
#define _BACKUP_H

#include <iostream>
#include <fstream>
#include <list>

#include "conf.h"
#include "new_process.h"
#include "../common/messages.h"
#include "restore.h"

using namespace std;

enum
  {
    PROCESS_CHAR_DEVICE,
    PROCESS_BLOCK_DEVICE,
    PROCESS_FIFO,
    PROCESS_SOCKET,
  };

class BU_Data

{
 public:
  string name;
  double stamp;
};

class Backup
  {
  friend class Frontend;        // Allow frontend to access everything
  friend class SlaveMessages;   // Allow slavemessages to access everything

  public:
    Backup(SlaveMessages *sm);
    ~Backup(void);

    void new_backup(string *conf_name);
    void update_backup(string *conf_name);

    static void get_file_data(string &pathname, list<File_Data> &data, bool orphans, SlaveMessages *m);

    static bool change_permissions_to_0777(string *name, SlaveMessages *m);
    static bool restore_permissions(string *name, struct stat64 *statbuf, SlaveMessages *m);
    static bool get_statbuf(string *name, struct stat64 *statbuf, SlaveMessages *m);
    static void fix_name(string &name, string *s);
    static bool get_target(string *name, string &target, SlaveMessages *m);

  private:
    bool create_directory(string *name);
    bool fix_conf(string *conf_name);
    void delete_old_backups(string *basename);
    bool delete_directory(string *dname);
    bool check_update_list(string *s_name);
    void save_attributes(string *s_name, struct stat64 *statbuf);
    bool set_dest_attributes(string *dest, bool link);

    bool add_timestamp_to_pathname(string *pathname);
    bool is_permitted(string *pathname);

    bool name_in_list(string *src, bool &newer);

    bool change_permissions_to_0755(string *name);

    bool process_name(string &dest, string &source, char *filename);
    bool process_source(string &source, string *current);
    void process_source_list(void);

    bool process_dir_contents(string *dir);
    bool process_file(string *d_file, string *s_file, struct stat64 *statbuf);
    bool process_link(string *d_link, string *s_link, struct stat64 *statbuf);
    bool process_node(string *d_node, string *s_node, struct stat64 *statbuf);
    bool mount_point_OK(void);
    void find_hostname(string *hostname);
    string &get_filename(string &pathname, string &filename);

    fstream errors_ofp;
    fstream data_ofp;
    string pathname;
    string basename;
    string data_pname;
    string errors_pname;
    string root_pname;

    bool update;

    list<File_Data> data;
    string compression;

    SlaveMessages *m;
    NewProcess run;
    Conf *conf;

  protected:
  };


#endif // _BACKUP_H
