/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _CONF_H
#define _CONF_H

#include <iostream>
#include <list>
#include <dirent.h>

using namespace std;

class Conf
{
friend class Slaveconf;  // Allow slaveconf to access everything
friend class Backup;     // Allow backup to access everything
friend class Restore;    // Allow backup to access everything

public:
  Conf(void);
  Conf(string &pathname);
  ~Conf(void);

  void load(string &pathname);
  void load(void);
  void save(string &pathname);
  void save(void);
  bool user_permitted(char *user);

  string log_type;
  string error_str;


private:
  void set_defaults(void);
  string &string_to_upper(string &str);
  bool IsSeparator(char x);
  void getright(string &s, string &v);

  string conf_pathname;


protected:
  list<string> users;
  list<string> backup_sources;
  list<string> backup_deny;

  string mount_point;
  string backup_destination;
  string compression;
  int keep_number;
  bool save_existing_files;
  bool restore_orphans;
  string sh_pname;
  string cat_pname;
  string gzip_pname;
  string bzip2_pname;
  string ln_pname;
  string mtab_pname;

  string browser_pname;

};
#endif //_CONF_H
