/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "new_process.h"
#include "conf.h"

NewProcess::NewProcess(void)
{
  // cout << "NewProcess:\n";
}

NewProcess::~NewProcess(void)
{
  // cout << "~NewProcess:\n";
}


// =============================================================
// NOTE:
//  system(command) is NBG because it loses suid priveleges
// This also uses /bin/sh which also may cause problems here
//
// The execve manual page says:
//  If the set-user-ID bit is set on the program file pointed to by
// filename, and the calling process is not being ptraced, then
// the effective user ID of the calling process is changed to that
// of the owner of the program file.  Similarly, when the set-group-ID
// bit of the program file is set the effective group ID of the calling
// process is set to the group of the program file.
//
// =============================================================
// Open a new process, command
// Add a pipe to deal with stderr output from the new process

int NewProcess::new_process(string &command, string &sh_pname)
{
extern char **environ;

int fd[2];
pid_t pid;
char c;
int n;

// NOTE:  pipes -  write into fp[1] and read from fp[0]


//cout << "NewProcess::new_process - command=  " << command  << endl;
//cout << "                        - sh_pname= " << sh_pname << endl;

status_string = "";

if (!(command.length()))
  {
  status_string = "Missing command";
  return(-1);   // COMMAND_ERR
  }
if (pipe(fd))
  {
  status_string = "Pipe error";
  return(-2);              // PIPE_ERR
  }

if ((pid = fork()) == -1)
  {
  status_string = "Fork error";
  return(-3);  // FORK_ERR
  }

if (pid == 0)                           // Child process
    {
      char *argv[4];
      argv[0] = "sh";
      argv[1] = "-c";
      argv[2] = (char *)command.c_str();
      argv[3] = 0;
     
      dup2(fd[1], 2);                     // duplicate stderr
      close(fd[0]);                       // Child is not reading (writing only)

      execve(sh_pname.c_str(), argv, environ);
      status_string = "Return error from execve";
      exit(-4);                          // RETURN_ERR - Error : it can't return!
    }

// I think we have to read the pipe here
close(fd[1]);                           // Parent is not writing (reading only)

// cout << "NewProcess::new_process - command=  " << command  << endl;

// Put this in a string to be accessed later if required

while ( (n = read(fd[0], &c, 1)) )
  status_string += c;

close(fd[0]);

for(;;)                                 // Parent process
  {
  int status = 0;

  if (waitpid(pid, &status, 0) == -1)
    {
    if (!(errno == EINTR))
      {
      status_string = "Not EINTR error";
      return(-5);   // NOT_EINTR_ERR
      }
    }
  else if (WIFEXITED(status))
    {
    status = (WEXITSTATUS(status));
    if (status == 0) status_string = "";
    return(status);                      // Command executed
    }
  else
    {
    // UNKNOWN ERROR
    status_string = "Unknown error";
    return(-6);         // UNKOWN_ERR
    }
  }
}



