/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _RESTORE_H
#define _RESTORE_H

#include <iostream>

#include <glib.h>

#include "new_process.h"
#include "../common/messages.h"
#include "conf.h"
#include "file_data.h"

using namespace std;

class Restore
{
  friend class SlaveMessages;   // Allow slavemessages to access everything

 public:
  Restore(string *from, string *config, SlaveMessages *sm);
  Restore(string *from, char *config, SlaveMessages *sm);
  ~Restore(void);

  static bool find_backup(string *pathname, Conf *conf, SlaveMessages *m);

 private:
    bool set_dest_attributes(string *dest, bool link);
    bool create_directory(string *name);
    void fix_uid_etc(void);
    bool name_in_list(string &pathname, list<File_Data>::iterator &p);

    bool get_parent_pathname(string *from, string *pathname);
    bool start_errors_file(string *restore_point, char *type, SlaveMessages *m);
    void get_compression(string *restore_point);
    void add_file_data(string &restore_point);
    
    void remove_compression_suffix(string &pathname, string &dest, string *fname, char *filename);
    void process_name(string &dest, string &source, char *filename);
    bool process_directory(string *d_dir, string *s_dir);
    bool process_file(string *d_file, string *s_file);
    bool same_target(string *name, string &target);
    bool process_link(string *d_link, string *s_link);
    bool process_node(string *d_node, string *s_node);
    bool files_identical(string *dest, string *src);

    NewProcess run;

    string compression;

    list<File_Data> data;
    list<list<File_Data>::iterator> data_pointer; // List of iterators for entries in "data" which were successfully restored

    class SlaveMessages *m;
    fstream errors_ofp;

    Conf *conf;

 protected:

};



#endif // _RESTORE_H
