/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>

#include <signal.h>
#include <sys/types.h>
#include <pwd.h>

#include "restore.h"
#include "conf.h"
#include "../common/messages.h"
#include "../common/misc.h"
#include "../common/version.h"
#include "slaveconf.h"

#define FOREVER for(;;)

using namespace std;
using namespace misc;

int main(int argc, char* argv[])
{
  pid_t spiffa_pid;
  pid_t slave_pid = getpid();
  string run_file = RUNDIR"/spiffaslave.pid";

  if (argc != 3) exit(1);


  spiffa_pid = atoi(argv[1]);

  Conf c;

  // send to spiffa, receive msgs for slave
  SlaveMessages m(atoi(argv[2]), spiffa_pid, slave_pid, &c.log_type);
  Slaveconf sc(m, c);
  m.put_slaveconf(&sc);

  // ============================================================
  // CHECK HERE for "/usr/local/spiffa/spiffaslave.pid"
  // ============================================================

  if (isfile((char *)run_file.c_str()))
    {
      string msg = "Spiffaslave - another copy of spiffaslave\n"
                    "is already running.\n"
                    "If this is not true, become root and delete\n"
                    "the file ";
      msg += "\"";
      msg += run_file;
      msg += "\".";
      m.send_message(SL_NO_PERMISSION, (char *)msg.c_str());
      return(0);
    }
  else
    {
      // Create the file
      fstream ofp;
      ofp.open(run_file.c_str(),
         ios::out | ios::binary | ios::trunc ); 
      if ( !ofp.is_open() )
        {
        // ERROR
        string msg = "Spiffaslave - can't open \"";
        msg += run_file;
        msg += "\" for write.";
        m.send_message(SL_NO_PERMISSION, (char *)msg.c_str());
        return(0);
        }
      ofp << slave_pid << endl;
      ofp.close();
      chown(run_file.c_str(), 0, 0);
      chmod(run_file.c_str(), 0600);
    }

  // Check that this user is permitted
  if (!c.user_permitted( (getpwuid( getuid() ))->pw_name   ))
    {
      string msg = "Permission denied for user \"";
      msg += (getpwuid( getuid() ))->pw_name;
      msg += "\".\nYou must be \"root\" to use this program.\n";
      msg += "From a terminal such as \"xterm\" or \"gnome-terminal\"\n";
      msg += "enter \"su\".  When prompted enter root's password\n";
      msg += "then enter \"spiffa\".\n\n";
      msg += "Alternatively, get the superuser to add you to the user list.";

      if (c.error_str.length())
        {
        msg += "\n\n";
        msg += c.error_str;
        }
      m.send_message(SL_NO_PERMISSION, (char *)msg.c_str());

      // REMOVE "/usr/local/spiffa/spiffaslave.pid" here
      remove(run_file.c_str());
      return(0);
    }

  // Tell spiffa the slave is alive!
  if (!m.send_message(SL_ALIVE, &slave_pid))
    {
      // REMOVE "/usr/local/spiffa/spiffaslave.pid" here
      remove(run_file.c_str());
      exit(1);
    }


  FOREVER
    {
      m.get_messages();  // if spiffa died, m.quit_flag is 1
      if (m.quit_flag) break;

      usleep(200);
    }

  // cout << "Spiffaslave - TERMINATING\n";
  


  // If this fails it shouldn't matter
  // but if restoring or saving completes before
  // spiffa's message is received, the GUI can freeze
  // waiting for the termination message
  m.send_message(SL_QUIT, NULL);

  // REMOVE "/usr/local/spiffa/spiffaslave.pid" here
  remove(run_file.c_str());
  
  //if (remove(run_file.c_str()))
  //  cout << "spiffaslave - main() FAILED to remove " << run_file.c_str() << endl;
  //else cout << "spiffaslave - main() REMOVED " << run_file.c_str() << endl;

  return 0;
}
