/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/types.h>
#include <pwd.h>

#include "slaveconf.h"
#include "../common/messages.h"
#include "conf.h"
#include "../common/misc.h"
#include "../common/tfile.h"

using namespace misc;


Slaveconf::Slaveconf(SlaveMessages &m, Conf &c)
{
send_conf_to_spiffa(m, c, TRUE);
}


Slaveconf::~Slaveconf(void)
{
  sources.clear();
  omissions.clear();
  users.clear();
}

void Slaveconf::send_conf_to_spiffa(SlaveMessages &m, Conf &c, bool want)
{
list<string>::iterator i;

if (want)
  {
  conf = &c;
  
  sources.clear();
  omissions.clear();
  users.clear();
  }
m.send_message(SL_CONF_START, NULL);

for (i=c.backup_sources.begin(); i!=c.backup_sources.end(); ++i)
  {
  m.send_message(SL_SOURCE, (char *)i->c_str());
  if (want) sources.push_back(*i);
  }

for (i=c.backup_deny.begin(); i!=c.backup_deny.end(); ++i)
  {
  m.send_message(SL_OMIT, (char *)i->c_str());
  if (want) omissions.push_back(*i);
  }

for (i=c.users.begin(); i!=c.users.end(); ++i)
  {
  if (STREQ((getpwuid( getuid() ))->pw_name, "root"))
    {
    m.send_message(SL_USER, (char *)i->c_str());
    }
    if (want) users.push_back(*i);
  }

if (want) mount_point = c.mount_point;
m.send_message(SL_MOUNT_POINT, (char *)c.mount_point.c_str());

if (want) backup_destination = c.backup_destination;
m.send_message(SL_DESTINATION, (char *)c.backup_destination.c_str());

if (want) compression = c.compression;
m.send_message(SL_COMPRESSION, (char *)c.compression.c_str());

if (want) keep_number = c.keep_number;
m.send_message(SL_KEEP_NUMBER, (char *)(TFile::itoa(c.keep_number)).c_str());

if (want) save_existing_files = c.save_existing_files;
m.send_message(SL_SAVE_EXISTING_FILES, (char *)((c.save_existing_files) ? "1" : "0") );

if (want) restore_orphans = c.restore_orphans;
m.send_message(SL_RESTORE_ORPHANS, (char *)((c.restore_orphans) ? "1" : "0") );

if (want)
  {
  sh_pname = c.sh_pname;
  cat_pname = c.cat_pname;
  gzip_pname = c.gzip_pname;
  bzip2_pname = c.bzip2_pname;
  ln_pname = c.ln_pname;
  mtab_pname = c.mtab_pname;
  browser_pname = c.browser_pname;
  }

if (STREQ((getpwuid( getuid() ))->pw_name, "root"))
  {
  m.send_message(SL_SH_PN, (char *)c.sh_pname.c_str());
  m.send_message(SL_CAT_PN, (char *)c.cat_pname.c_str());
  m.send_message(SL_GZIP_PN, (char *)c.gzip_pname.c_str());
  m.send_message(SL_BZIP2_PN, (char *)c.bzip2_pname.c_str());
  m.send_message(SL_LN_PN, (char *)c.ln_pname.c_str());
  m.send_message(SL_MTAB_PN, (char *)c.mtab_pname.c_str());
  }

// Spiffa uses this one anyway
m.send_message(SL_BROWSER_PN, (char *)c.browser_pname.c_str());

if (want) log_type = c.log_type;
m.send_message(SL_LOG_TYPE, (char *)c.log_type.c_str());

m.send_message(SL_CONF_DONE, NULL);
}



string *Slaveconf::save_conf(string *conf_file)
{
  list<string>::iterator i;

  // Send the configuration to the spiffa.conf file
  conf->backup_sources.clear();
  for (i=sources.begin(); i!=sources.end(); ++i)
    {
      conf->backup_sources.push_back(*i);
    }

  conf->backup_deny.clear();
  for (i=omissions.begin(); i!=omissions.end(); ++i)
    {
      conf->backup_deny.push_back(*i);
    }

  conf->users.clear();
  for (i=users.begin(); i!=users.end(); ++i)
    {
      conf->users.push_back(*i);
    }

  conf->mount_point = mount_point;
  conf->backup_destination = backup_destination;
  conf->compression = compression;
  conf->keep_number = keep_number;
  conf->save_existing_files = save_existing_files;
  conf->restore_orphans = restore_orphans;

  conf->sh_pname = sh_pname;
  conf->cat_pname = cat_pname;
  conf->gzip_pname = gzip_pname;
  conf->bzip2_pname = bzip2_pname;
  conf->ln_pname = ln_pname;
  conf->mtab_pname = mtab_pname;
  conf->browser_pname = browser_pname;

  conf->log_type = log_type;

  // Now save it!
  if (conf_file->length())
    {
      conf->save(*conf_file);
    }
  else
    {
      conf->save();
    }
  return &conf->error_str;
}
