/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <string>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <stdlib.h>
#include <pwd.h>
#include <signal.h>
#include <errno.h>

#include "../common/messages.h"
#include "slaveconf.h"
#include "../common/misc.h"
#include "../common/tfile.h"
#include "../common/version.h"
#include "backup.h"
#include "restore.h"

using namespace std;
using namespace misc;

SlaveMessages::SlaveMessages(int q, int mst, int mrt, string *log_type):Messages(q, mst)
{
  // cout << "SlaveMessages:\n";
  mrtype = mrt;
  error_fp = NULL;
  want_log = TRUE;
  quit_flag = 0;

  // The choice is NEW (trunc) APPEND (app) or NONE
  if (*log_type == "NEW")
    log_fp.open(LOGDIR"/spiffa.log", ios::out | ios::binary | ios::trunc );
  else if (*log_type == "APPEND")
    log_fp.open(LOGDIR"/spiffa.log", ios::out | ios::binary | ios::app );
  else want_log = FALSE;
  if (log_fp.is_open()) log_fp.close();
}

SlaveMessages::~SlaveMessages(void)
{
  // cout << "~SlaveMessages:\n";
  log_fp.close();
}


void SlaveMessages::get_messages(void)
{
  int len;
  struct spiffa_msg msg;
  

  // CORRECT pid
  //cout << "SlaveMessages::get_messages - mstype= " << mstype << endl;

  if (kill(mstype, 0)) // BUT THIS says spiffa has died!
    {
      quit_flag = 1; // spiffa is dead
      return;
    }

  while ( (len = msgrcv(qid, &msg, MBUFSIZE, mrtype, IPC_NOWAIT)) >= 0 )
    {
      switch (msg.dtype)
  {
  case SP_START_SOURCES:
    sconf->sources.clear();
    sconf->backup_destination = "";
  break;

  case SP_SOURCE:
    sconf->sources.push_back((char *)msg.data);
  break;

  case SP_START_OMISSIONS:
    sconf->omissions.clear();
  break;

  case SP_OMIT:
    sconf->omissions.push_back((char *)msg.data);
    break;

  case SP_START_USERS:
    if (STREQ ( (getpwuid( getuid() ))->pw_name, "root" ) )
      {
        sconf->users.clear();
      }
  break;

  case SP_USER:
    if (STREQ ( (getpwuid( getuid() ))->pw_name, "root" ) )
      {
      sconf->users.push_back((char *)msg.data);
      }
  break;

  case SP_MOUNT_POINT:
    sconf->mount_point = (char *)msg.data;
  break;

  case SP_DESTINATION:
    sconf->backup_destination = (char *)msg.data;
  break;

  case SP_COMPRESSION:
    sconf->compression = (char *)msg.data;
  break;

  case SP_KEEP_NUMBER:
    sconf->keep_number = atoi((char *)msg.data);
  break;

  case SP_SAVE_EXISTING_FILES:
    sconf->save_existing_files =  (STREQ((char *)msg.data, "1" ) ? TRUE : FALSE);
  break;

  case SP_RESTORE_ORPHANS:
    sconf->restore_orphans = (STREQ((char *)msg.data, "1" ) ? TRUE : FALSE);
  break;

  case SP_SH_PN:
    sconf->sh_pname = (char *)msg.data;
  break;

  case SP_CAT_PN:
    sconf->cat_pname = (char *)msg.data;
  break;

  case SP_GZIP_PN:
    sconf->gzip_pname = (char *)msg.data;
  break;

  case SP_BZIP2_PN:
    sconf->bzip2_pname = (char *)msg.data;
  break;

  case SP_LN_PN:
    sconf->ln_pname = (char *)msg.data;
  break;

  case SP_MTAB_PN:
    sconf->mtab_pname = (char *)msg.data;
  break;

  case SP_BROWSER_PN:
    sconf->browser_pname = (char *)msg.data;
  break;

  case SP_LOG_TYPE:
    sconf->log_type = (char *)msg.data;
  break;

  case SP_CONF_DONE:
    {
    string error_str;
    string conf_file = (char *)msg.data;
    error_str = *(sconf->save_conf(&conf_file));
    send_message(SL_SAVE_DONE, (char *)error_str.c_str());
    }
  break;

  case SP_BACKUP:
    {
    char *action = "New Backup";
    Backup b(this);
    string conf_file = (char *)msg.data;

    start_action(action);
    b.new_backup(&conf_file);
    finish_action(action);
    }
  break;

  case SP_UPDATE:
    {
    char *action = "Update Backup";
    Backup b(this);
    string conf_file = (char *)msg.data;

    start_action(action);
    b.update_backup(&conf_file);
    finish_action(action);
    }
  break;

  case SP_START_RESTORE_SELECTION:
  restore_selection.clear();
  break;

  case SP_RESTORE_SELECTION_NAME:
    {
    File_Data d;
    d.backup_name = (char *)msg.data;
    restore_selection.push_back(d);
//  restore_selection.push_back( (char *)msg.data );
    }
  break;

  case SP_END_RESTORE_SELECTION:
/*****************************************************
// OK HERE
{
list<string>::iterator i;

cout << "SlaveMessages::get_messages  - THE LIST" << endl;

for (i=restore_selection.begin(); i!=restore_selection.end(); ++i)
  {
  cout << *i << endl;
  }
cout << "===============================\n";
}
*****************************************************/

  {
  char *action = "Restore Selection";
  string from = "";
  string conf = "";

  char *p = (char *)msg.data;

   if (p)
    {
    while (*p && (*p != '\n'))
        {
        from += *p;
        p++;
        }
      if (*p == '\n')
        {
        p++;
        while (*p && (*p != '\n'))
            {
            conf += *p;
            p++;
            }
        }
    }
  
  start_action(action);
  Restore r(&from, (char *)(conf.c_str()), this);
  finish_action(action);
  }
  break;

  case SP_RESTORE_FROM:
    {
    char *action = "Restore";
    string from = "";
    string conf = "";

    char *p = (char *)msg.data;

    if (p)
      {
      while (*p && (*p != '\n'))
        {
        from += *p;
        p++;
        }
      if (*p == '\n')
        {
        p++;
        while (*p && (*p != '\n'))
          {
          conf += *p;
          p++;
          }
        }
      }
    restore_selection.clear();
    start_action(action);
    Restore r(&from, &conf, this);
    finish_action(action);
    }
  break;


  case SP_LOAD:
    if ( (char *)msg.data && (*(char *)msg.data) )
      {
      string pathname = (char *)msg.data;
      Conf c = Conf(pathname);

      if (c.error_str.length())
        {
        send_message(SL_CONF_DONE, (char *)c.error_str.c_str() );
        }
      else sconf->send_conf_to_spiffa(*this, c, FALSE);
      }
    break;

  case SP_QUIT:
    // cout << "SlaveMessages: GOT SP_QUIT" << endl;
    quit_flag = 2;  // User Abort
    break;

  default:
    // cout << "SlaveMessages: UNKNOWN MESSAGE, DATA TYPE " << msg.dtype << endl;
    break;  // Ignore unknown messages
  }
    }
}

void SlaveMessages::put_slaveconf(class Slaveconf *sc)
{
  sconf = sc;
}
void SlaveMessages::put_error_fp(fstream *fp)
{
  error_fp = fp;
}


void SlaveMessages::log_message(string &msg)
{
if (msg[msg.length()-1] != '\n') msg += "\n";
if (want_log)
    {
    log_fp.open(LOGDIR"/spiffa.log", ios::out | ios::binary | ios::app );
    }
if (log_fp.is_open())
    {
    log_fp << msg;
    log_fp.close();
    }
if (STREQN((char *)msg.c_str(), "FATAL", 5))
    {
    send_message(SL_FATAL_ERROR, (char *)msg.c_str());
    }
else if ( (STREQN((char *)msg.c_str(), "ERROR", 5)) ||
      (STREQN((char *)msg.c_str(), "WARNING", 7)) )
    {
    send_message(SL_TEXT, (char *)msg.c_str());
    }
cerr << msg;
}

void SlaveMessages::write_to_errors(string &msg)
{
if (msg[msg.length()-1] != '\n') msg += "\n";

if ( (error_fp) && error_fp->is_open() )
    {    
    *error_fp << msg;
    }
log_message(msg);
}

void SlaveMessages::process_warning(string &msg)
{
  string error = "WARNING: " + msg;
  write_to_errors(error);
  num_warnings++;
}
void SlaveMessages::process_error(string &msg)
{
  string error = "ERROR: " + msg;
  write_to_errors(error);
  num_errors++;
}
void SlaveMessages::process_fatal_error(string &msg)
{
  string error = "FATAL ERROR: " + msg;

  write_to_errors(error);

  num_errors++;
}

int SlaveMessages::get_num_errors(void)
{
  return num_errors;
}

bool SlaveMessages::start_action(char *action)
{
string msg;

num_errors = num_warnings = num_names = num_skipped = 0;
start_time = time(NULL);
if (start_time == -1)
  {
  msg = "Can't get the time for ";
  msg += action;
  msg += "! : ";
  msg += strerror(errno);
  process_fatal_error(msg);
  return FALSE;
  }

else
  {
  msg = "\n";
  msg += version_txt;
  msg += "\n";
  msg += action;
  msg += ": ";
  msg += ctime(&start_time);
  msg += "User: ";
  msg += (getpwuid( getuid() ))->pw_name;
  log_message(msg);
  }
return TRUE;
}


void SlaveMessages::finish_action(char *action)
{
string msg;
string elapsed_str;
string time_now;

time_t t = time(NULL);

switch (quit_flag)
    {
    case 0:
    // normal exit
    break;

    case 1:
    msg = "\"spiffa\" has quit or died";
    break;

    case 2:
    msg = "User Abort!";
    break;

    default:
    msg = "UNKNOWN";
    }
if (quit_flag)
    {
    msg += " : \"spiffaslave\" is terminating";
    process_warning(msg);
    }


if (t == -1)
  {
  msg = "Can't get the time at finish of ";
  msg += action;
  msg += "! : ";
  msg += strerror(errno);
  process_error(msg);
  time_now = "time FAILED!";
  elapsed_str = "UNKNOWN!";
  }
else
  {
  time2string( (t - start_time), elapsed_str);
  time_now = ctime(&t);
  time_now = time_now.substr(0, time_now.length() - 1);
  }

msg = "FINISHED ";
msg += action;
msg += ": ";
msg += time_now;
msg += " : Elapsed Time ";
msg += elapsed_str;
msg += "\nErrors ";
msg += TFile::itoa(num_errors);
msg += " : Warnings ";
msg += TFile::itoa(num_warnings);
msg += " : Names Processed ";
msg += TFile::itoa(num_names);

if (action=="Update Backup")
  {
  msg += " (of which ";
  msg += TFile::itoa(num_skipped);
  msg += " were skipped)";
  }

send_message(SL_DONE_BACKUP, (char *)msg.c_str() );
write_to_errors(msg);
}




